#----------------------------------------------------------------------------
# $Copyright: Copyright (c) 2016 Veritas Technologies LLC.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# VERITAS TECHNOLOGIES LLC.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF VERITAS TECHNOLOGIES LLC.
#
# The  Licensed  Software  and  Documentation  are  deemed  to be commercial
# computer  software  as  defined  in  FAR  12.212 and subject to restricted
# rights  as  defined in FAR Section 52.227-19 "Commercial Computer Software
# - Restricted  Rights"  and  DFARS 227.7202,  et seq.  "Commercial Computer
# Software  and  Commercial Computer Software Documentation," as applicable,
# and any successor regulations, whether delivered by Veritas as on premises
# or  hosted  services.  Any  use,  modification,  reproduction  release,
# performance,  display  or  disclosure  of  the  Licensed  Software  and
# Documentation by the U.S. Government shall be solely in accordance with
# the terms of this Agreement. $
#----------------------------------------------------------------------------
# $Header: /project/isv-agent/ACCLib/lib/system/Proc.pm,v 1.28 2014/10/10 07:02:21 lpatil Exp $
#
# Veritas Software Corporation
#
# $Author: lpatil $
# $Date: 2014/10/10 07:02:21 $
# $Revision: 1.28 $
#
# Provides: package Proc
#
# Functionality: Process and PID file
#
# Authors: 011787
#----------------------------------------------------------------------------

#----------------------------------------------------------------------------
# General info:
#
# There are 2 formats for holding process information,
# i)  PidList - simple array of process Ids (ints)
# ii) ProcInfoHashRefList - array of hash referances 
#     format of hash
#
#     "UID"     =>  <user id>
#     "PID"     =>  <process id>
#     "PPID"    =>  <parent process id>
#     "COMMAND" =>  <command string>
#----------------------------------------------------------------------------

#----------------------------------------------------------------------------
# HP-UX changes
# add the UNIX95 env setting in GetProcessListHash so hp's 'ps' looks like 
# Solaris. change GetPidsStateFromList so it uses a 'ps' listing to see whats
# online (rather than 'if(-d "/proc/$iEachPid/fd")'
#----------------------------------------------------------------------------

#---------------------
# version calls.
#---------------------
use Version;
Version::TellVersion("\$Header: /project/isv-agent/ACCLib/lib/system/Proc.pm,v 1.28 2014/10/10 07:02:21 lpatil Exp $::ETVHeader");
#----------------------
# end of version calls.
#----------------------

#-----------------------------
# Declare the package name.
#-----------------------------
package Proc;

#-------------------------------------------------
# Export the constants needed for IPC resources.
#-------------------------------------------------
require Exporter;
@ISA = qw(Exporter);
@EXPORT = qw(O_IPCSHM O_IPCSEM O_IPCMSQ O_IPCALL);

#----------------------------------------
# turn on strict for refs, subs and vars.
#----------------------------------------
use strict;

#------------------------------------
# Include all the modules required.
#------------------------------------
use LogInt;        # For logging
use VCSDefines;    # For online / partial online / offline
use Sys;           # For deleting pidfiles
use Arch;          # For Platform Determination

use IO::File;
use IO::Handle;
use IPC::SysV qw ( IPC_STAT IPC_RMID );
use POSIX qw ( uname );

#-----------------------
# Define IPC constants..
#-----------------------
use constant O_IPCSHM => 1 << 0;                     # Shared memory..
use constant O_IPCSEM => 1 << 1;                     # Semaphore..
use constant O_IPCMSQ => 1 << 2;                     # Message Queue..
use constant O_IPCALL => O_IPCSHM|O_IPCSEM|O_IPCMSQ; # All three..

#---------------------------------
# Initialize the globals required.
#---------------------------------
$::gbOSType        = Arch::GetOSType();
$::gsCommandIpcs   = "/usr/bin/ipcs";
$::gsCommandPs     = "/bin/ps";
$::gsCommandArgsPs = "-e -o 'uid,pid,ppid,args'";

if ( $::gbOSType == Arch::HPUX_OS ) {
  $::gsCommandArgsPs = "-ex -o 'uid,pid,ppid,args'";
} 
elsif ( $::gbOSType == Arch::LINUX_OS ) {
  $::gsCommandArgsPs = "-ew -o 'uid,pid,ppid,args'";
}

#------------------------------------------------------------
# time to wait between kills and checking if kill worked.
#------------------------------------------------------------
$::gsKillWait = 5;

#---------------------------------
# const defines for 'ProcType'.
#---------------------------------
use constant PROC_TYPE_PARENT       => 1;
use constant PROC_TYPE_CHILDREN     => 2;
use constant PROC_TYPE_ALL          => 3;


=pod

----------------------------------------------------------------------------
Function: Boolean ReadPidFile( IN String PidFile, 
                               OUT ListOfInteger Pids )

Description:
  Reads the all the process IDs from the given file and puts them in a list.

Parameters:
  PidFile : File name containing the process IDs.
  Pids    : The list of process IDs.

Return Value:
  Boolean value in the set of {0,1} where
         0 - false
         1 - true
----------------------------------------------------------------------------

=cut

sub ReadPidFile {

  my ( $sPidFile,
       $rliPids ) = @_;

  my $sMyName = (caller 0)[3];

  LogInt::Message( LogInt::LLTrace,
  10024,
  "Proc",
  "ReadPidFile",
  "Subroutine <%s> is called with:
  PID File        [%s]",
  $sMyName, $sPidFile );

  if(open(PFILE, "$sPidFile")) {
    @$rliPids = <PFILE>;
    close(PFILE);
    chomp @$rliPids;
    return 1;
  }
  else {
    LogInt::Message( LogInt::LLError,
                     10025,
                     "Proc",
                     "ReadPidFile",
                     "ReadPidFile failed to open filename [%s] for reading",
                     $sPidFile );
    return 0; 
  }
}


=pod

----------------------------------------------------------------------------
Function: Boolean WritePidFile( OUT String PidFile,
                                IN ListOfInteger Pids )

Description:
  Writes the process IDs(PID) into the file.

Parameters:
  PidFile : File containing the PID once it writes to a file.
  Pids    : List of process IDs that are needs to be written into the file.

Return Value:
  Boolean value in the set of {0,1} where
        0 - false
        1 - true
----------------------------------------------------------------------------

=cut

sub WritePidFile {

  my ( $sPidFile,
       $rliPids ) = @_;

  my $sMyName = (caller 0)[3];

  LogInt::Message( LogInt::LLTrace, 
  10026,
  "Proc",
  "WritePidFile",
  "Subroutine <%s> is called with:
  Filename          [%s]", 
  $sMyName, $sPidFile );

  if(open(PFILE, ">$sPidFile")) {
    foreach my $iEachPid (@$rliPids) {
      LogInt::Message( LogInt::LLTrace, 
                       10027,
                       "Proc", 
                       "WritePidFile",
                       "Writing PID:[%s] to PidFile:[%s]", 
                       $iEachPid,
                       $sPidFile );
      print PFILE "$iEachPid\n";
    }
    close(PFILE);
    return 1;
  }
  else {
    LogInt::Message( LogInt::LLError, 
                     10028,
                     "Proc",
                     "WritePidFile",
                     "WritePidFile failed to open filename [%s] for writing",
                     $sPidFile );
    return 0; #return with error
  }
}


=pod

-------------------------------------------------------------------------
Function: Integer GetDefunctProcessListHash( OUT ArrayOfHashRefs Procs )

Description:
  Gets all the defunct processes running.

Parameters:
  Procs : A list of defunct processes. 
          Each defunct process has an entry in the array.
          Each entry in the list is a reference to hasH with keys
          UID, PID, PPID, COMMAND.

Return Value:
  Returns number of defunct processes found.
-------------------------------------------------------------------------

=cut

sub GetDefunctProcessListHash {
  
  my ( $rlhrProcs ) = @_;

  my $sMyName = (caller 0)[3];

  LogInt::Message( LogInt::LLTrace, 
  10029, 
  "Proc",
  "GetDefunctProcessListHash",
  "Subroutine <%s> called", $sMyName );

  LogInt::Message( LogInt::LLTrace, 
                   10030, 
                   "Proc",
                   "GetDefunctProcessListHash",
                   "Calling ps command [%s] with options [%s]",
                   $::gsCommandPs,
                   "-ef" );

  #--------------------------------------
  # empty the hash array (just incase).
  #--------------------------------------
  @$rlhrProcs = ();

  #-------------------------------------
  # from 'man ps' exit values are:
  # 0     Successful completion.
  # >0    An error occurred.
  #-------------------------------------

  #-------------------------------------------------------------------
  # On hpux we can set an ENV variable (UNIX95) to give us our normal 
  # PS command.
  #-------------------------------------------------------------------
  my $iIsSetUNIX95 = exists $ENV{"UNIX95"};
  if($iIsSetUNIX95) {
    delete $ENV{"UNIX95"}; 
  } 

  my $sCmdArgs = "-ef";
  my @lsProcs  = `$::gsCommandPs $sCmdArgs 2>&1`;
  my $iReturnCode = $? >> 8;

  if($iIsSetUNIX95) {
    #set it back
    $ENV{"UNIX95"} = 1;
  } 

  #-------------------------------------------------------------
  # check the return code if not 0 then print an error message 
  # using the returned string and return.
  #-------------------------------------------------------------
  if ($iReturnCode != 0) {
    my $sErrorString;
    if(@lsProcs > 1) {
      $sErrorString = join('\n', @lsProcs);
    } 
    else {
      $sErrorString = $lsProcs[0];
    }
    chomp $sErrorString;
    LogInt::Message( LogInt::LLError,
                     10031, 
                     "Proc",
                     "GetDefunctProcessListHash",
                     "Failed with message [%s]",
                     $sErrorString );
    return 0; #exit number of proc's zero - wouldnt happen generally - error
  }

  #----------------------------------------
  # put all the proc's into a hash array.
  #
  # format of hash
  #
  # "UID"     =>  <user id>
  # "PID"     =>  <process id>
  # "PPID"    =>  <parent process id>
  # "COMMAND" =>  <command string>
  #----------------------------------------

  chomp @lsProcs;
  shift @lsProcs; #remove the PS header

  my $iTmpUID     = undef;
  my $iTmpPID     = undef;
  my $iTmpPPID    = undef;
  my $sTmpCOMMAND = undef;

  my $sOSType = Arch::GetOSType();

  foreach my $sEachProc (@lsProcs) {
    if( $sEachProc =~ /<defunct>/ ) {
      my %hTmpProcHash = ();

      if( $sOSType == Arch::LINUX_OS ) {
        if($sEachProc =~ /^\s*([a-zA-Z0-9-._]+)\s+([0-9]+)\s+([0-9]+)\s+.*?\b[0-9][0-9]:[0-9][0-9]:[0-9][0-9]\b\s+(.+)$/) {
          my $sUser    = $1;
          my $iUid     = getpwnam( $sUser );
          $iTmpUID     = $iUid;
          $iTmpPID     = $2;
          $iTmpPPID    = $3;
          $sTmpCOMMAND = "<defunct>";
        }
      }
      else {
        if($sEachProc =~ /^\s*([a-zA-Z0-9-._]+)\s+([0-9]+)\s+([0-9]+)\s+.*?\b[0-9]:[0-9][0-9]\b\s+(.+)$/) {
          my $sUser    = $1;
          my $iUid     = getpwnam( $sUser );
          $iTmpUID     = $iUid;
          $iTmpPID     = $2;
          $iTmpPPID    = $3;
          $sTmpCOMMAND = "<defunct>"; 
        }
      }

      $hTmpProcHash{"UID"}     = $iTmpUID;
      $hTmpProcHash{"PID"}     = $iTmpPID;
      $hTmpProcHash{"PPID"}    = $iTmpPPID;
      $hTmpProcHash{"COMMAND"} = $sTmpCOMMAND;

      LogInt::Message( LogInt::LLTraceTrace,
                       10032,
                       "Proc",
                       "GetDefunctProcessListHash",
                       "UID:[%s] PID:[%s] PPID[%s] COMMAND[%s]",
                       $iTmpUID,
                       $iTmpPID,
                       $iTmpPPID,
                       $sTmpCOMMAND );

      #-----------------------------------
      # Insert hash reference into list.
      #-----------------------------------
      push(@$rlhrProcs, \%hTmpProcHash);
    }
  }

  my $iNumberOfPIDs = @$rlhrProcs;

  LogInt::Message( LogInt::LLTrace,
                   10033,
                   "Proc", 
                   "GetDefunctProcessListHash",
                   "Got [%s] defunct processes", 
                   $iNumberOfPIDs );

  return $iNumberOfPIDs;
}


=pod

----------------------------------------------------------------------------
Function: Integer GetProcessListHash( OUT ListOfHashRefs Procs )

Description:
  Collects all the processes currently running on the system.

Parameters:
  Procs : An array of hashes containing all the processes 
                        currently running on the system.

Return Value:
  Returns number of processes in the list.
----------------------------------------------------------------------------

=cut

sub GetProcessListHash {
  return Proc::GetLongProcessListHash ( @_ );
}

#-------------
# Deprecated..
#-------------
sub __GetProcessListHash {

  my ( $rlhrProcs ) = @_;

  my $sMyName = (caller 0)[3];

  LogInt::Message( LogInt::LLTrace, 
  10034, 
  "Proc",
  "GetProcessListHash",
  "Subroutine <%s> called", $sMyName );

  LogInt::Message( LogInt::LLTrace, 
                   10035, 
                   "Proc",
                   "GetProcessListHash",
                   "Calling ps command [%s] with options [%s]",
                   $::gsCommandPs,
                   $::gsCommandArgsPs );

  #--------------------------------------
  # empty the hash array (just incase).
  #--------------------------------------
  @$rlhrProcs = ();

  #-------------------------------------
  # from 'man ps' exit values are:
  # 0     Successful completion.
  # >0    An error occurred.
  #-------------------------------------

  #-------------------------------------------------------------------
  # On hpux we can set an ENV variable (UNIX95) to give us our normal 
  # PS command.
  #-------------------------------------------------------------------
  my $iIsSetUNIX95 = exists $ENV{"UNIX95"};
  if($iIsSetUNIX95) {
    #leave it set
  } 
  else {
    #set it ourselves
    $ENV{"UNIX95"} = 1;
  }

  #---------------------------------------------------
  # Get all defunct processes from process table.
  #---------------------------------------------------
  my @lhrProcs = ();

  if( GetDefunctProcessListHash(\@lhrProcs) ) {
	@$rlhrProcs = @lhrProcs;
  }
  else {
    LogInt::Message( LogInt::LLTraceTrace,
                     10036, 
                     "Proc",
                     "GetProcessListHash",
                     "No defunct processes are found" ); 
  }
  my @lsProcs     = `$::gsCommandPs $::gsCommandArgsPs 2>&1`;
  my $iReturnCode = $? >> 8;

  if($iIsSetUNIX95) {
    #leave it set
  } 
  else {
    #unset it ourselves
    delete $ENV{"UNIX95"};
  }

  #-------------------------------------------------------------
  # check the return code if not 0 then print an error message 
  # using the returned string and return.
  #-------------------------------------------------------------
  if ($iReturnCode != 0) {
    my $sErrorString;
    if(@lsProcs > 1) {
      $sErrorString = join('\n', @lsProcs);
    } 
    else {
      $sErrorString = $lsProcs[0];
    }
    chomp $sErrorString;
    LogInt::Message( LogInt::LLError,
                     10037, 
                     "Proc",
                     "GetProcessListHash",
                     "Failed with message [%s]",
                     $sErrorString );
    return 0; #exit number of proc's zero - wouldnt happen generally - error
  }

  #----------------------------------------
  # put all the proc's into a hash array.
  #
  # format of hash
  #
  # "UID"     =>  <user id>
  # "PID"     =>  <process id>
  # "PPID"    =>  <parent process id>
  # "COMMAND" =>  <command string>
  #----------------------------------------

  chomp @lsProcs;
  shift @lsProcs; #remove the PS header

  foreach my $sEachProc (@lsProcs) {
    #------------------------------------------------------------------------
    #Format is
    # <space>* uid <space>+ pid <space>+ ppid <space>+ args [args<space>+]*
    #------------------------------------------------------------------------
    if($sEachProc =~ /^\s*([0-9]+)\s+([0-9]+)\s+([0-9]+)\s+(.+)$/) {
      my $iTmpUID     = $1;
      my $iTmpPID     = $2;
      my $iTmpPPID    = $3;
      my $sTmpCOMMAND = $4;

      LogInt::Message( LogInt::LLTraceTrace,
                       10038,
                       "Proc",
                       "GetProcessListHash",
                       "UID:[%s] PID:[%s] PPID[%s] COMMAND[%s]",
                       $iTmpUID,
                       $iTmpPID,
                       $iTmpPPID,
                       $sTmpCOMMAND );

      #----------------------
      # Insert into hash.
      #----------------------
      my %hTmpProcHash;
      $hTmpProcHash{"UID"}     = $iTmpUID;
      $hTmpProcHash{"PID"}     = $iTmpPID;
      $hTmpProcHash{"PPID"}    = $iTmpPPID;
      $hTmpProcHash{"COMMAND"} = $sTmpCOMMAND;

      #-----------------------------------
      # Insert hash reference into list.
      #-----------------------------------
      push(@$rlhrProcs, \%hTmpProcHash);
    }
  }

  my $iNumberOfPIDs = @$rlhrProcs;

  LogInt::Message( LogInt::LLTrace,
                   10039,
                   "Proc", 
                   "GetProcessListHash",
                   "Got [%s] processes", 
                   $iNumberOfPIDs );

  return $iNumberOfPIDs;
}


=pod

----------------------------------------------------------------------------
Function: Integer GetLongProcessListHash( OUT ListOfHashRefs Procs,
                                          IN  Boolean        EnvFlag )

Description:
  Gets all the processes currently running on the system.

Parameters:
  $rlhrProcs : An array of hashes containing all the processes 
              currently running on the system.
  $bEnvFlag  : Get environment of running processes (if OS is not HP-UX) 

Return Value:

  Returns the number of processes in list with long command output.
----------------------------------------------------------------------------

=cut

sub GetLongProcessListHash {

  my ( $rlhrProcs,
       $bEnvFlag ) = @_;

  $bEnvFlag ||= 0; # Prevent undefs..

  my $sMyName = ( caller 0 )[3];

  LogInt::Message( LogInt::LLTrace,
  10040, 
  "Proc",
  "GetLongProcessListHash",
  "Subroutine <%s> called with:
   rlhrProcs        [%s]
   EnvFlag          [%s]" ,
   $sMyName, $rlhrProcs, $bEnvFlag );

  #----------------------------------
  # Prevent env-flag usage on HP-UX..
  #----------------------------------
  if ( $^O eq 'hpux' && $bEnvFlag ) {
    LogInt::Message ( LogInt::LLError,
                      10041,
                      "Proc",
                      "GetLongProcessListHash",
                      "EnvFlag [%s] not supported on OS [%s]",
                      $bEnvFlag, uc $^O );
    return 0; # Error..
  }

  #-------------------------------------
  # empty the hash array (just incase).
  #-------------------------------------
  @$rlhrProcs = ();

  my $sEnv = '';

  if ( $bEnvFlag ) {
    $sEnv = $^O eq 'aix' ? 'we' : 'e'; # Set environment request..
  }  

  my $sCommandPs     = $::gsCommandPs;
  my $sCommandPsArgs = "axww${sEnv}l";
  my $bOSType        = Arch::GetOSType();
  my @lsFieldNames   = ();

  #--------------------------------------
  # from 'man ps' exit values are:
  # 0     Successful completion.
  # >0    An error occurred.
  #--------------------------------------

  if ( Arch::SUN_OS == $bOSType ) {
    my $sOSVer = ( uname ) [2]; # Get OS version..
    if ($sOSVer =~ /\d+\.(\d+)/) {
      $sOSVer =$1;
    }
    $sCommandPs = "/usr/bin/ps";
    if ( $sOSVer < 11 ) {
      $sCommandPs = "/usr/ucb/ps" if ( -f "/usr/ucb/ps" );
    }
  }

  elsif ( Arch::HPUX_OS == $bOSType ) {
    $sCommandPsArgs = "-e -xxo uid,pid,ppid,args";
  }
  elsif ( Arch::LINUX_OS == $bOSType ) {
    $sCommandPsArgs = "ww${sEnv} -eo uid,pid,ppid,args"
  } 

  LogInt::Message( LogInt::LLTrace,
                   10042, 
                   "Proc",
                   "GetLongProcessListHash",
                   "Calling ps command [%s] with options [%s]",
                   $sCommandPs,
                   $sCommandPsArgs );

  my $iIsSetUNIX95 = exists $ENV { UNIX95 };
  $ENV { UNIX95 } = 1 unless ( $iIsSetUNIX95 );

  #------------------
  # Set LANG to "C"..
  #------------------
  my $iIsSetLANG = $ENV { LANG };
  $ENV { LANG } = 'C';

  #---------------------------------------------------
  # Get all defunct processes from process table.
  #---------------------------------------------------
  my @lhrProcs = ();
  my @lsProcs = `$sCommandPs $sCommandPsArgs`;
  my $iReturnCode = $? >> 8;

  unless ( $iIsSetUNIX95 ) {
    delete $ENV { UNIX95 };
  }

  $iIsSetLANG ? $ENV { LANG } = $iIsSetLANG : delete $ENV { LANG };

  #----------------------------------------------------------------------
  # Check the return code if not 0 then print an error message using 
  # the returned string and return.
  #----------------------------------------------------------------------
  if ($iReturnCode != 0) {
    my $sErrorString = '';
    if( scalar @lsProcs > 1) {
      $sErrorString = join('\n', @lsProcs);
    }
    else {
      $sErrorString = $lsProcs[0];
    }
    chomp $sErrorString;
    LogInt::Message( LogInt::LLError,
                     10043,
                     "Proc", 
                     "GetLongProcessListHash",
                     "Failed with message [%s]", 
                     $sErrorString );
    return 0; #exit number of proc's zero - wouldnt happen generally - error
  }

  #-------------------------------------------
  # put all the proc's into a hash array.
  # format of hash.
  #
  # "UID"     =>  <user id>
  # "PID"     =>  <process id>
  # "PPID"    =>  <parent process id>
  # "COMMAND" =>  <command string>
  #-------------------------------------------

  shift @lsProcs; #remove the PS header

  my @lsDet = (); # Process details..
  my %hsOSReg = (
    'solaris'   => '[\da-z]+\s+([\d]+)\s+([\d]+)\s+([\d]+)\s+.*?[\d]:[\d]{2}\s(.*)',
    'aix'       => '[\da-e]+\s+[A-Z]\s+([\d]+)\s+([\d]+)\s+([\d]+)\s+.*?[\d]:[\d]{2}\s(.*)',
    'hpux'      => '([\d]+)\s+([\d]+)\s+([\d]+)\s+(.*)',
    'linux'     => '([\d]+)\s+([\d]+)\s+([\d]+)\s+(.*)' 
  );
  
  #------------------------
  # Set the correct regex..
  #------------------------
  my $sFmt = $hsOSReg { $^O };
  
  foreach my $sEachProc (@lsProcs) {
    $sEachProc =~ s/\n//;   # Chomp..
    $sEachProc =~ s/^\s+//; # Strip leading spaces..
    @lsDet = ( $1, $2, $3, $4 ) if ( $sEachProc =~ /^$sFmt$/o );

    LogInt::Message( LogInt::LLTraceTrace,
                     10044,
                     "Proc", 
                     "GetLongProcessListHash",
                     "UID [%s] PID [%s] PPID [%s] COMMAND [%s]", 
                     $lsDet [0], $lsDet [1], $lsDet [2], $lsDet [3] );

    #-------------------
    # Insert into hash..
    #-------------------
    my %hTmpProcHash = ();
    if ( 4 == scalar @lsDet ) {
      @hTmpProcHash { qw ( UID PID PPID COMMAND ) } = @lsDet;
      push ( @$rlhrProcs, \%hTmpProcHash ); # Insert hash reference into list..
    }
    else {
      #-----------------------------------
      # Produce error output for debug.
      #-----------------------------------
      LogInt::Message( LogInt::LLTrace, 
                       10045,
                       "Proc",
                       "GetLongProcessListHash",
                       "Error Parsing ps output:[%s]",
                       $sEachProc );
    }
  }

  my $iNumberOfPIDs = scalar @$rlhrProcs;
  LogInt::Message( LogInt::LLTrace, 
                   10046, 
                   "Proc", 
                   "GetLongProcessListHash",
                   "Got [%s] processes",
                   $iNumberOfPIDs );

  return $iNumberOfPIDs;
}


=pod

----------------------------------------------------------------------------
Function: Integer FilterProcs( IN String Filter, 
                               IN ListOfHashRefs Procs,
                               OUT ListOfHashRefs MatchedProcs )

Description:
  Filters all the processes from the process list and collects all the 
  matched processes into another hash.

Parameters:
  Filter       : The string used to filter the processes.
  Procs        : The list of processes that needs to be filtered out.
  MatchedProcs : Processes remained after filtering with the filter.

Return Value:
  Returns count of matched processes
----------------------------------------------------------------------------

=cut

sub FilterProcs {

  my ( $sFilter,
       $rlhrProcs,
       $rlhrMatchedProcs ) = @_;

  my $sMyName = (caller 0)[3];

  LogInt::Message( LogInt::LLTrace,
  10047,
  "Proc",
  "FilterProcs",
  "Subroutine <%s> called with:
  Filter          [%s]",
  $sMyName, $sFilter );

  #--------------------------------------------------------------------------
  # previous incarnation of this function was concerned against accidental
  # matches of $sFilter with agent scripts - we are only going to use full
  # paths of commands when matching.
  #--------------------------------------------------------------------------
  #---------------------------------------
  # format of hash
  #
  # "UID"     =>  <user id>
  # "PID"     =>  <process id>
  # "PPID"    =>  <parent process id>
  # "COMMAND" =>  <command string>
  #---------------------------------------

  foreach my $hrEachProcHashRef (@$rlhrProcs) {
    my $sTmpCOMMAND = $$hrEachProcHashRef{"COMMAND"};

    if(! defined($sTmpCOMMAND) or ($sTmpCOMMAND eq "") ) {
      # not valid resource. Skip.
    }
    elsif($sTmpCOMMAND =~ /$sFilter/) {
      LogInt::Message( LogInt::LLTrace, 
                       10048,
                       "Proc", 
                       "FilterProcs",
                       "Matched command [%s]",
                       $sTmpCOMMAND );
      push(@$rlhrMatchedProcs, $hrEachProcHashRef);
    }

  } #end for each proc

  my $iNumberMatches = @$rlhrMatchedProcs;

  LogInt::Message( LogInt::LLTrace, 
                   10049,
                   "Proc",
                   "FilterProcs",
                   "Got [%s] matches",
                   $iNumberMatches );

  return $iNumberMatches;
}


=pod

----------------------------------------------------------------------------
Function: Integer FindUidPids( IN Integer UID,
                               IN ListOfHashRefs Procs, 
                               OUT ListOfHashRefs MatchedProcs )

Description: 
  Finds all the process IDs of all the processes belongs to a PID.

Parameters:
  UID          : User IDentifier.
  Procs        : Array of process hashs from used to get the pids.
  MatchedProcs : List of pids matched with UID.

Return Value:
  Returns the number of matched processes.
----------------------------------------------------------------------------

=cut

sub FindUidPids {

  my ( $UID,
       $rlhrProcs, 
       $rlhrMatchedProcs ) = @_;

  my $sMyName = (caller 0)[3];

  LogInt::Message( LogInt::LLTrace,
  10050,
  "Proc",
  "FindUidPids",
  "Subroutine <%s> called with:
  UID           [%s]",
  $sMyName, $UID );

  #--------------------------------------
  # format of hash
  #
  # "UID"     =>  <user id>
  # "PID"     =>  <process id>
  # "PPID"    =>  <parent process id>
  # "COMMAND" =>  <command string>
  #--------------------------------------

  foreach my $hrEachProcHashRef (@$rlhrProcs) {
    my $sTmpUID = $$hrEachProcHashRef{"UID"};

    if ( ! defined($UID) || ! defined($sTmpUID) ||
            ($UID eq "") || ($sTmpUID eq "") ) {
      # Uid is invalid. Don't test further.
    }
    else {
      if ( $UID =~ /\D/) { 
        #--------------------------------------------------
        # UID is not a valid number. It contains some non 
        # digits too. Workaround detected.
        #--------------------------------------------------
        if($sTmpUID =~ /$UID/) { 
          LogInt::Message( LogInt::LLTrace,
                           10051,
                           "Proc",
                           "FindUidPids",
                           "Matched UID [%s]",
                           $sTmpUID );
          push(@$rlhrMatchedProcs, $hrEachProcHashRef);
        }
      }
      else {
        #-----------------------------
        # UID is a valid number.
        #-----------------------------
        if($sTmpUID == $UID) {
          LogInt::Message( LogInt::LLTrace, 
                           10052,
                           "Proc",
                           "FindUidPids",
                           "Matched UID [%s]", 
                           $sTmpUID );
          push(@$rlhrMatchedProcs, $hrEachProcHashRef);
        }
      }
    }
  } #end for each proc

  my $iNumberMatches = @$rlhrMatchedProcs;

  LogInt::Message( LogInt::LLTrace,
                   10053, 
                   "Proc",
                   "FindUidPids",
                   "Got [%s] matches",
                   $iNumberMatches );

  return $iNumberMatches;
}



=pod

----------------------------------------------------------------------------
Function: Constant GetPidsStateFromList( IN ListOfIntegers Pids, 
                                         OUT ListOfIntegers OnlinePids, 
                                         OUT ListOfIntgers OfflinePids, 
                                         IN Boolean LongCommand )

Description:
  Separate the online, offline process from the list of given processes.

Parameters:
  Pids        : List of PIDs for which status needs to be checked.
  OnlinePids  : List of Process IDs which are online.
  OfflinePids : List of process IDs which are offline.
  LongCommand : Flag to specify the process list type (i.e., long or short).

Return Value:
  Returns one of the monitor exit codes:
          ONLINE
          OFFLINE
          PARTIAL_ONLINE
----------------------------------------------------------------------------

=cut

sub GetPidsStateFromList {
 
  my ( $rliPids,
       $rliOnlinePids, 
       $rliOfflinePids, 
       $bLongCommand ) = @_;

  my $sMyName = (caller 0)[3];
  my $liPidList = join(' ', @$rliPids);

  LogInt::Message( LogInt::LLTrace, 
  10054,
  "Proc",
  "GetPidsStateFromList",
  "Subroutine <%s> called with:
  Pids       [%s]",
  $sMyName, $liPidList );

  if (!defined $bLongCommand) {
    $bLongCommand = 0;
  }
  else { 
    $bLongCommand = 1;
  }

  #-----------------------
  # clear incoming lists.
  #-----------------------
  @$rliOnlinePids  = ();
  @$rliOfflinePids = ();

  my $iOfflineCount = 0;
  my $iOnlineCount  = 0;
  my $iProcessCount = 0;

  #----------------------------------------------------------------
  # get a list of all the PIDs which exist in the process table.
  #----------------------------------------------------------------

  #-----------------------------------------------------
  # Call PS and store in hash. Added in $bLongCommand 
  # flag to specify if we need long ps command output.
  #-----------------------------------------------------
  my @hProcListHash = ();
  if ($bLongCommand == 0) {
    $iProcessCount = Proc::GetProcessListHash(\@hProcListHash);
  } 
  else {
    $iProcessCount = Proc::GetLongProcessListHash(\@hProcListHash);
  }

  #-----------------------------------
  # check how many procs were found.
  #-----------------------------------
  if($iProcessCount == 0) {
    #-----------------------------------
    # log error and return offline.
    #-----------------------------------
    LogInt::Message( LogInt::LLError,
                     10055,
                     "Proc", 
                     "GetPidsStateFromList",
                     "Call to get process list did not return any processes" );
    return VCSDef_MonitorExit::OFFLINE;
  } 
  else {
    LogInt::Message( LogInt::LLTrace,
                     10056, 
                     "Proc", 
                     "GetPidsStateFromList",
                     "Call to get process list returned [%s] processes", 
                     $iProcessCount );
  }

  my @liPidList = ();
  Proc::GetValuesForKeyFromProcDataHash("PID", \@hProcListHash, \@liPidList);

  #------------------------------------------------------------------------
  # make a hash slice of all the PIDs which exist in the process table.
  #------------------------------------------------------------------------
  my %hsAllPidsHashSlice = ();
  @hsAllPidsHashSlice { @liPidList } = (1) x @liPidList;

  #------------------------------------------------------------------------------
  # spin the list of pids we got passed an see if they exist in the hash slice.
  #------------------------------------------------------------------------------
  foreach my $iEachPid (@$rliPids) {
    if(defined $hsAllPidsHashSlice{$iEachPid}) {
      push(@$rliOnlinePids, $iEachPid);
      $iOnlineCount++;
      LogInt::Message( LogInt::LLTrace, 
                       10057, 
                       "Proc",
                       "GetPidsStateFromList",
                       "Checking PID [%s] State:Active",
                       $iEachPid );
    } 
    else {
      push(@$rliOfflinePids, $iEachPid);
      $iOfflineCount++;
      LogInt::Message( LogInt::LLTrace,
                       10058,
                       "Proc",
                       "GetPidsStateFromList",
                       "Checking PID [%s] State:InActive",
                       $iEachPid );
    }
  }

  LogInt::Message( LogInt::LLTrace,
                   10059,
                   "Proc",
                   "GetPidsStateFromList",
                   "Online pid count [%s], offline pid count [%s]", 
                   $iOnlineCount, 
                   $iOfflineCount );

  #---------------------------------------------------------------------
  # if the PID list is empty (both counts are zero) we return 'offline'.
  #---------------------------------------------------------------------
  if($iOfflineCount > 0) {
    if($iOnlineCount > 0) {
      LogInt::Message( LogInt::LLTrace,
                       10060, 
                       "Proc",
                       "GetPidsStateFromList",
                       "Returning Status [PARTIAL_ONLINE]" );
      return VCSDef_MonitorExit::PARTIAL_ONLINE;
    }
    else {
      LogInt::Message( LogInt::LLTrace, 
                       10061,
                       "Proc",
                       "GetPidsStateFromList",
                       "Returning Status [OFFLINE]" );
      return VCSDef_MonitorExit::OFFLINE;
    }
  }
  else {
    if($iOnlineCount > 0) {
      LogInt::Message( LogInt::LLTrace,
                       10062,
                       "Proc",
                       "GetPidsStateFromList",
                       "Returning Status [ONLINE]" );
      return VCSDef_MonitorExit::ONLINE;
    } 
    else {
      LogInt::Message( LogInt::LLTrace,
                       10063,
                       "Proc",
                       "GetPidsStateFromList",
                       "Returning Status [OFFLINE]" );
      return VCSDef_MonitorExit::OFFLINE;
    }
  }
}


=pod

----------------------------------------------------------------------------
Function: Boolean GetPidsStateFromFile( IN String PidFile, 
                                        OUT ListOfIntegers OnlinePids, 
                                        OUT ListOfIntegers OfflinePids, 
                                        IN Boolean LongCommand )

Description:
  Gets the process ID states in the file.

Parameters:
  PidFile     : File containg the process IDs.
  OnlinePids  : List of process IDs which are online.
  OfflinePids : List of process IDs which are offline.
  LongCommand : Flag to specify the process list type (i.e., short or long). 

Return Value:
  Boolean value in the set of {0,1} where
         0 - false
         1 - true
----------------------------------------------------------------------------

=cut

sub GetPidsStateFromFile {

  my ( $sPidFile,
       $rliOnlinePids,
       $rliOfflinePids,
       $bLongCommand ) = @_;

  my $sMyName = (caller 0)[3];

  LogInt::Message( LogInt::LLTrace,
  10064, 
  "Proc",
  "GetPidsStateFromFile",
  "Subroutine <%s> called with:
  PidFile         [%s]",
  $sMyName, $sPidFile );

  if (!defined $bLongCommand ) {
    $bLongCommand = 0;
  } 
  else {
    $bLongCommand = 1;
  }

  my @liPids;
  if(ReadPidFile($sPidFile, \@liPids)) {
    LogInt::Message( LogInt::LLTrace,
                     10065, 
                     "Proc",
                     "GetPidsStateFromFile",
                     "Calling GetPidsStateFromList" );
    return GetPidsStateFromList(\@liPids, $rliOnlinePids, $rliOfflinePids, $bLongCommand);
  } 
  else {
    LogInt::Message( LogInt::LLError,
                     10066, 
                     "Proc",
                     "GetPidsStateFromFile",
                     "Error in call to ReadPidFile" );
    return 0;
  }
}



=pod

----------------------------------------------------------------------------
Function: Boolean KillPidsFromList( IN ListOfIntegers Pids,
                                    IN Boolean LongCommand )

Description:
  Kills all the processes whose PID is in the list.

Parameters:
  Pids        : List of PIDs to be killed.
  LongCommand : A flag to indicate to get the long list of processes.

Return Value:
  Boolean value in the set of {0,1} where
         0 - false
         1 - true
----------------------------------------------------------------------------

=cut

sub KillPidsFromList {

  my ( $rliPids, 
       $bLongCommand ) = @_;

  my $sMyName = (caller 0)[3];
  my $liPidList = join(' ', @$rliPids);

  LogInt::Message( LogInt::LLTrace, 
  10067,
  "Proc",
  "KillPidsFromList",
  "Subroutine <%s> called with:
  Pids          [%s]",
  $sMyName, $liPidList );

  #--------------------------------------------------------------------------
  # 2003-07-28: tmortens - Added in $bLongCommand flag to specify if we 
  # need long ps command output.
  # To support older agents not specifying if it needs long Command output 
  # from ps. also added $bLongCommand flag to the end each 
  # call of GetPridsStateFromList.
  #---------------------------------------------------------------------------
  if(! defined $bLongCommand) {
    $bLongCommand = 0;
  }
  else {
    $bLongCommand = 1;
  }

  #--------------------------------------------------------------
  # assign this to a local copy so we can change it if needs be.
  #--------------------------------------------------------------
  my @liPidsToKill = @$rliPids;

  #-----------------------------------------------------------------------------
  # firstly see if 'our' pid is in the list - ie $$
  # dprice - moved code up (logically) to remove pid from list 2002-03-02
  # 2002-01-31: kbarth - 
  # Check that you don't kill yourself!... Don't laugh; this is a fairly 
  # common occurrence when agents are written that grep for a string in
  # the process table, and that string matches a portion of the resource name,
  # as it often would as designers like to keep related 
  # software objects similarly named....
  #-----------------------------------------------------------------------------

  #-----------------------------------------------
  # make a hash slice of the liPidsToKill list.
  #-----------------------------------------------
  my %hsTmpTestThisProcessPidHash = ();
  @hsTmpTestThisProcessPidHash { @liPidsToKill } = (1) x @liPidsToKill;
  my $iThisProcessPid = $$;

  if(defined $hsTmpTestThisProcessPidHash{$iThisProcessPid}) {
    #---------------------------------------------------------
    # 'we' are in the list - need to take it out and reassign 
    # the list w/o ThisProcessPid in it.
    #---------------------------------------------------------
    delete $hsTmpTestThisProcessPidHash{$iThisProcessPid}; #remove key/value

    #--------------------------------------------------------------
    # assign remaining pid's back into the list of pids to kill.
    #--------------------------------------------------------------
    @liPidsToKill = keys %hsTmpTestThisProcessPidHash; 
    LogInt::Message( LogInt::LLTrace,
                     10068,
                     "Proc",
                     "KillPidsFromList",
                     "Avoided killing PID [%s] (it's our own PID)",
                     $iThisProcessPid);
  }

  my @liOnlinePids  = ();
  my @liOfflinePids = ();

  my $iPidsState = GetPidsStateFromList( \@liPidsToKill, 
                                         \@liOnlinePids,
                                         \@liOfflinePids,
                                         $bLongCommand );

  if(VCSDef_MonitorExit::OFFLINE == $iPidsState) {
    #----------------------------------------
    # pids already offline - return good.
    #----------------------------------------
    LogInt::Message( LogInt::LLTrace,
                     10069, 
                     "Proc", 
                     "KillPidsFromList",
                     "Pids were offline in first check" );
    return 1;
  }

  foreach my $iEachPidToKill (@liOnlinePids) {
    LogInt::Message( LogInt::LLTrace,
                     10070,
                     "Proc", 
                     "KillPidsFromList",
                     "Sending signal [%s] to PID [%s]",
                     "TERM",
                     $iEachPidToKill );
    kill "TERM", $iEachPidToKill;
  }

  LogInt::Message( LogInt::LLTrace, 
                   10071, 
                   "Proc", 
                   "KillPidsFromList",
                   "Sleeping for [%s] seconds", 
                       $::gsKillWait );

  sleep $::gsKillWait;

  my @liOnlinePidsAfterTerm   = ();
  my @liOfflinePidsAfterTerm  = ();

  my $iPidsStateAfterTerm = GetPidsStateFromList( \@liOnlinePids,
                                                  \@liOnlinePidsAfterTerm,
                                                  \@liOfflinePidsAfterTerm,
                                                  $bLongCommand );

  if(VCSDef_MonitorExit::OFFLINE == $iPidsStateAfterTerm) {
    #-----------------------------------
    # pids now offline - return good.
    #-----------------------------------
    LogInt::Message( LogInt::LLTrace,
                     10072,
                     "Proc",
                     "KillPidsFromList",
                     "Pids were offline after kill (TERM)" );
    return 1;
  }

  foreach my $iEachPidToKill (@liOnlinePidsAfterTerm) {
    LogInt::Message( LogInt::LLTrace,
                     10073,
                     "Proc", 
                     "KillPidsFromList",
                     "Sending signal [%s] to PID [%s]","KILL", 
                     $iEachPidToKill );
    kill "KILL", $iEachPidToKill;
  }

  LogInt::Message( LogInt::LLTrace, 
                   10074,
                   "Proc", 
                   "KillPidsFromList",
                   "Sleeping for [%s] seconds", 
                   $::gsKillWait );

  sleep $::gsKillWait;

  my @liOnlinePidsAfterKill  = ();
  my @liOfflinePidsAfterKill = ();

  my $iPidsStateAfterKill = GetPidsStateFromList( \@liOnlinePidsAfterTerm,
                                                  \@liOnlinePidsAfterKill, 
                                                  \@liOfflinePidsAfterKill,
                                                  $bLongCommand );

  if(VCSDef_MonitorExit::OFFLINE == $iPidsStateAfterKill) {
    #-----------------------------------
    # pids now offline - return good.
    #-----------------------------------
    LogInt::Message( LogInt::LLTrace,
                     10075,
                     "Proc", 
                     "KillPidsFromList",
                     "Pids were offline after kill (KILL)" );
    return 1;
  } 
  else {
    #----------------------------------------------------
    # failed to kill the pids - log warn - return error.
    #----------------------------------------------------
    LogInt::Message( LogInt::LLWarn, 
                     10076,
                     "Proc",
                     "KillPidsFromList",
                     "Failed to kill Pids after kill (KILL)" );
    return 0;
  }
}


=pod

----------------------------------------------------------------------------
Function: Boolean KillPidsFromFile( IN String PidFile, 
                                    IN Boolean LongCommand )

Description:
  Kills all the process ID from the file. 

Parameters:
  PidFile     : Name of the file containing the process IDs.
  LongCommand : Flag to indicate to get the long proces list. 

Return Value:
  Boolean value in the set of {0,1} where
         0 - false
         1 - true
----------------------------------------------------------------------------

=cut

sub KillPidsFromFile {

  my ( $sPidFile,
       $bLongCommand ) = @_;

  my $sMyName = (caller 0)[3];

  LogInt::Message( LogInt::LLTrace,
  10077,
  "Proc",
  "KillPidsFromFile",
  "Subroutine <%s> called with:
  PidFile        [%s]",
  $sMyName, $sPidFile );

  if(!defined $bLongCommand) {
    $bLongCommand = 0; 
  }
  else {
    $bLongCommand = 1;
  }

  my @liPids = ();
  if(ReadPidFile($sPidFile, \@liPids)) {
    LogInt::Message( LogInt::LLTrace,
                     10078,
                     "Proc",
                     "KillPidsFromFile",
                     "Calling KillPidsFromList" );
    return KillPidsFromList( \@liPids, $bLongCommand );
  } 
  else {
    LogInt::Message( LogInt::LLError, 
                     10079,
                     "Proc", 
                     "KillPidsFromFile",
                     "Error in call to ReadPidFile" );
    return 0;
  }

}


=pod

----------------------------------------------------------------------------
Function: Integer GetChildPidsForGivenPids( IN ListOfIntegers Pids,
                                            OUT ListOfIntegers ChildPids ) 
                                  

Description:
  Get all the process child PIDs for given Process IDs (PID). 

Parameters:
  Pids       : PIDs of the processes for which children processes are required.
  ChildPids : List of children pids for the given pids.

Return Value:
  Integer : Returns the number of children proceses found.
----------------------------------------------------------------------------
=cut

sub GetChildPidsForGivenPids {

  my ( $rliPids,
       $rliChildPids ) = @_;

  my $sMyName = (caller 0)[3];
  my $liPidList = join(' ', @$rliPids);

  LogInt::Message( LogInt::LLTrace,
  10080,
  "Proc",
  "GetChildPidsForGivenPids", 
  "Subroutine <%s> called with:
  PIDs            [%s]",
  $sMyName, @$rliPids );

  my @lhrProcListHash = ();
                                                                                
  if( Proc::GetLongProcessListHash(\@lhrProcListHash) ) {
    foreach my $iPid (@$rliPids ) {
      foreach my $sEachChild ( @lhrProcListHash) {
        if ( $sEachChild->{"PPID"} == $iPid ) {
          LogInt::Message( LogInt::LLTraceTrace,
                           10081,
                           "Proc",
                           "GetChildPIDsForGivenPID",
                           "[%s] child is found for [%s]", $sEachChild->{"PID"}, $iPid );
          push (@$rliChildPids , $sEachChild->{"PID"});
        }
      }
    }
  }
  my $iNumOfChilds = scalar @$rliChildPids;
  if( $iNumOfChilds ) {
    LogInt::Message( LogInt::LLTrace,
                     10082,
                     "Proc",
                     "GetChildPIDsForGivenPID",
                     "[%s] children found for the given PIDs [%s]", $iNumOfChilds, @$rliPids );
  }
  else {
    LogInt::Message( LogInt::LLTrace,
                     10083,
                     "Proc",
                     "GetChildPIDsForGivenPID",
                     "No children found for the given PIDs [%s]", @$rliPids );
  }

  return $iNumOfChilds;
}


=pod

----------------------------------------------------------------------------
Function: Void SignalPidsFromList( OUT ListOfIntegers Pids, 
                                   IN String Signal )

Description:
  Send a signal to all the processes in the list.

Parameters:
  Pids   : List of processes needs to be send the signal.
  Signal : Type of the signal to send to the processes.

Return Value:
  Void - No value is returned to the caller.
----------------------------------------------------------------------------

=cut

sub SignalPidsFromList {

  my ( $rliPids,
       $sSignal ) = @_;

  my $sMyName = (caller 0)[3];
  my $liPidList = join(' ', @$rliPids);

  LogInt::Message( LogInt::LLTrace,
  10084,
  "Proc", 
  "SignalPidsFromList",
  "Subroutine <%s> called with:
  Pids         [%s]
  Signal       [%s]",
  $sMyName, $liPidList, $sSignal );

  foreach my $iEachPidToSignal (@$rliPids) {
    #----------------------------------------------------------------------- 
    # 2002-01-31: kbarth - 
    # Check that you don't kill yourself!... Don't laugh; this is a fairly
    # common occurrence when agents 
    # are written that grep for a string in the process table, and that 
    # string matches a portion of the resource name, as it often would 
    # as designers like to keep related 
    # software objects similarly named....
    #----------------------------------------------------------------------
    if( $iEachPidToSignal != $$ ) {
      LogInt::Message( LogInt::LLTrace,
                       10085,
                       "Proc", 
                       "SignalPidsFromList",
                       "Sending signal [%s] to PID [%s]",
                       $sSignal,
                       $iEachPidToSignal );

      kill $sSignal, $iEachPidToSignal;
    } 
    else {
      LogInt::Message( LogInt::LLTrace,
                       10086,
                       "Proc", 
                       "SignalPidsFromList",
                       "Avoided sending signal [%s] to PID [%s] (it's our own PID)", 
                       $sSignal,
                       $iEachPidToSignal );
    }
  }
  
  return;
}


=pod

----------------------------------------------------------------------------
Function: Boolean CheckProcApplicationPidFile( IN String PidFile, 
                                               IN Boolean LongCommand )

Description:
  Checks the status of the process IDs of the 
  application which are given in the file.

Parameters:
  PidFile     : Name of the file containing process IDs of the application.
  LongCommand : Flag to specify to use long command or normal.

Return Value:
  Boolean value in the set of {0,1} where
         0 - false
         1 - true
----------------------------------------------------------------------------

=cut

sub CheckProcApplicationPidFile {
 
  my ( $sPidFile, 
       $bLongCommand ) = @_;

  my $sMyName = (caller 0)[3];

  LogInt::Message( LogInt::LLTrace,
  10087, 
  "Proc", 
  "CheckProcApplicationPidFile",
  "Subroutine <%s> called with:
  PidFile        [%s]", 
  $sMyName, $sPidFile );

  if(!defined $bLongCommand) {
    $bLongCommand = 0;
  }
  else {
    $bLongCommand = 1;
  }

  my @lsTmpOnlinePids  = ();
  my @lsTmpOfflinePids = ();

  return GetPidsStateFromFile( $sPidFile, 
                               \@lsTmpOnlinePids,
                               \@lsTmpOfflinePids,
                               $bLongCommand );
}


=pod

----------------------------------------------------------------------------
Function: Integer CheckProcMaintainedPidFile(
                                    IN ListOfStrings ProcNameFilters, 
                                    IN Integer NumberOfEachProcRequired,
                                    IN String ProcType, 
                                    IN String PidFile, 
                                    IN Boolean LongCommand )

Description:
  Checks the Pid file for existence. If exists it check the processes
  status, otherwise it will gets all the processes with the specified 
  filters and checks the status.

Parameters:
  ProcNameFilters          : List containing the filters.
  NumberOfEachProcRequired : Specifies the number of processes required
                             for each type.
  ProcType                 : Type of the processes required.
                             The following process types are allowed.
                                 PROC_TYPE_CHILDREN
                                 PROC_TYPE_PARENT
                                 PROC_TYPE_ALL
  PidFile                  : Name of the file containing process IDs.
  LongCommand              : Flag to specify to use long or normal command 
                             to get the process IDs.

Return Value:
  Returns one of the monitor exit status:
          ONLINE
          OFFLINE
          PARTIAL_ONLINE or 
          0 for error.
----------------------------------------------------------------------------

=cut

sub CheckProcMaintainedPidFile {

  my ( $rlsProcNameFilters,
       $iNumberOfEachProcRequired,
       $ProcType,
       $sPidFile, 
       $bLongCommand ) = @_;

  my $sMyName = (caller 0)[3];

  LogInt::Message( LogInt::LLTrace,
  10088,
  "Proc",
  "CheckProcMaintainedPidFile",
  "Subroutine <%s> called with:
  PidFile        [%s]", 
  $sMyName, $sPidFile );

  if(!defined $bLongCommand) {
    $bLongCommand = 0;
  }
  else {
    $bLongCommand = 1;
  }

  #-----------------------------
  # see if PidFile exists.
  #-----------------------------
  if(Sys::FileExists($sPidFile)) {
    #----------------------------------------------------------
    # file exists - use it to see if all the pids are online.
    # get pid file data.
    #----------------------------------------------------------
    my @lsTmpOnlinePids  = ();
    my @lsTmpOfflinePids = ();
    my $iPidState = GetPidsStateFromFile( $sPidFile,
                                          \@lsTmpOnlinePids,
                                          \@lsTmpOfflinePids,
                                          $bLongCommand );

    if(VCSDef_MonitorExit::ONLINE == $iPidState) {
      #-------------------------------
      # pids all ok - return good.
      #-------------------------------
      LogInt::Message( LogInt::LLTrace, 
                       10089,
                       "Proc",
                       "CheckProcMaintainedPidFile",
                       "All pids online in first check returning status [ONLINE]" );
      return VCSDef_MonitorExit::ONLINE;

    } 
    else {
      #-----------------------------------------------------------------
      # some (or all) pids are having a problem - or the PidFile didnt 
      # exist perhaps delete the pidfile its no use to us at the moment.
      #-----------------------------------------------------------------
      if(Sys::DeleteFile($sPidFile)) {
        LogInt::Message( LogInt::LLTrace,
                         10090,
                         "Proc", 
                         "CheckProcMaintainedPidFile",
                         "Deleted PidFile [%s]",
                         $sPidFile );
      } 
      else {
        #--------------------------------------------------------
        # if we fail to delete the PidFile then exit with error.
        #--------------------------------------------------------
        LogInt::Message( LogInt::LLError,
                         10091,
                         "Proc",
                         "CheckProcMaintainedPidFile",
                         "Failed to delete PidFile [%s]",
                         $sPidFile );
        return 0; #exit with error 
      }
    }
  }
  #-------------------------------------------------------
  # if we're here then we have no pid file
  # get the 'ps' data then filter it using our ProcFilters
  #-------------------------------------------------------
  my @lhrProcessDataList = ();

  if($bLongCommand == 0) {
    if(GetProcessListHash(\@lhrProcessDataList) > 0) {
      #got more than zero proc's - continue
    } 
    else {
      LogInt::Message( LogInt::LLError,
                       10092,
                       "Proc",
                       "CheckProcMaintainedPidFile",
                       "GetProcessListHash returned no proc's" );
      return 0; #exit with error 
    }
  }
  else {
    if(GetLongProcessListHash(\@lhrProcessDataList) > 0) {
      #got more than zero proc's - continue
    } 
    else {
      LogInt::Message( LogInt::LLError, 
                       10093,
                       "Proc",
                       "CheckProcMaintainedPidFile",
                       "GetLongProcessListHash returned no proc's" );
      return 0; #exit with error 
    }

  }

  my @lhrMatchedProcessDataList = ();

  #-----------------------------------------------
  # spin the passed in list of rlsProcNameFilters.
  #-----------------------------------------------
  foreach my $sEachFilter (@$rlsProcNameFilters) {
    my @lhrTmpMatchedProcessDataList;

    if( FilterProcs( $sEachFilter,
                     \@lhrProcessDataList,
                     \@lhrTmpMatchedProcessDataList ) < $iNumberOfEachProcRequired ) {
      #--------------------------------------------------------------------
      # we can jump here - regardless of anything else - we didnt even get
      # iNumberOfEachProcRequired matches.
      #--------------------------------------------------------------------
      LogInt::Message( LogInt::LLTrace,
                       10094,
                       "Proc",
                       "CheckProcMaintainedPidFile",
                       "Filter loop got less than [%s] matches returning status [OFFLINE]",
                       $iNumberOfEachProcRequired );
      return VCSDef_MonitorExit::OFFLINE;
    }

    if((PROC_TYPE_CHILDREN == $ProcType) || (PROC_TYPE_PARENT == $ProcType)) {
      my @lhrParents  = ();
      my @lhrChildren = ();
      SeparateOutChildrenAndParents( \@lhrTmpMatchedProcessDataList,
                                     \@lhrParents,
                                     \@lhrChildren );
      if(PROC_TYPE_CHILDREN == $ProcType) {
        if(@lhrChildren < $iNumberOfEachProcRequired) {
          LogInt::Message( LogInt::LLTrace,
                           10095,
                           "Proc",
                           "CheckProcMaintainedPidFile",
                           "Filter loop got less than [%s] child matches returning status [OFFLINE]",
                           $iNumberOfEachProcRequired );
          return VCSDef_MonitorExit::OFFLINE;
        }
        else {
          #-------------------------------------------------------
          # add the children matches for this filter pass to the 
          # total list of matches so far
          #-------------------------------------------------------
          push(@lhrMatchedProcessDataList, @lhrChildren);
        }
      } 
      else {
        if(@lhrParents == 0) {
          #--------------------------------------------------------------------------
          # because parents are defined in relation to children - if 
          # we have no parents we have no children
          #
          # we already know that the FilterProcs found >= iNumberOfEachProcRequired
          # we are taking a view that if there are no child parent relations 
          # (because no parents) then all entries will be taken as being parents 
          # - ie the test has passed - use lhrTmpMatchedProcessDataList
          #---------------------------------------------------------------------------
          LogInt::Message( LogInt::LLTrace, 
                           10096, 
                           "Proc",
                           "CheckProcMaintainedPidFile",
                           "SeparateOutChildrenAndParents found no parent-child relationships (OK)");
          #--------------------------------------------------------
          # add the all the matches for this filter pass to the 
          # total list of matches so far
          #--------------------------------------------------------
          push(@lhrMatchedProcessDataList, @lhrTmpMatchedProcessDataList);
        } 
        elsif (@lhrParents < $iNumberOfEachProcRequired) {
          #------------------------------------------------
          # didnt meet the criteria - return 'offline'.
          #------------------------------------------------
          LogInt::Message( LogInt::LLTrace, 
                           10097,
                           "Proc", 
                           "CheckProcMaintainedPidFile",
                           "Filter loop got less than [%s] parent matches returning status [OFFLINE]", 
                           $iNumberOfEachProcRequired );
          return VCSDef_MonitorExit::OFFLINE;
        }
        else {
          #------------------------------------------------------
          # add the parent matches for this filter pass to the 
          # total list of matches so far
          #------------------------------------------------------
          push(@lhrMatchedProcessDataList, @lhrParents);
        }
      }
    }
    else {
      #-------------------------------------------------------------------------------
      #PROC_TYPE_ALL - disregard any parent/child relationships
      #add all the matches for this filter pass to the total list of matches so far.
      #-------------------------------------------------------------------------------
      push(@lhrMatchedProcessDataList, @lhrTmpMatchedProcessDataList);
    }
  } #end of filter loop

  #---------------------------------------------------------------------
  # change the lhrMatchedProcessDataList into Pids and write to file.
  #---------------------------------------------------------------------
  my @liPidList;
  GetValuesForKeyFromProcDataHash("PID", \@lhrMatchedProcessDataList, \@liPidList);

  #---------------------------------------------------------------
  # write the pids to the file - if this fails then log warning 
  # - but we still return online.
  #---------------------------------------------------------------
  if(WritePidFile($sPidFile, \@liPidList)) {
    #all ok continue
  } 
  else {
    LogInt::Message( LogInt::LLWarn,
                     10098, 
                     "Proc",
                     "CheckProcMaintainedPidFile",
                     "Failed in call to WritePidFile with PidFile [%s]",
                     $sPidFile );
  }

  LogInt::Message( LogInt::LLTrace,
                   10099,
                   "Proc", 
                   "CheckProcMaintainedPidFile",
                   "Returning Status [ONLINE]" );

  return VCSDef_MonitorExit::ONLINE;
}


=pod

----------------------------------------------------------------------------
Function: Integer CheckProcProcessList( IN ListOfString ProcNameFilters, 
                                        IN Integer NumberOfEachProcRequired, 
                                        IN String ProcType, 
                                        IN Boolean LongCommand)

Description:
  Checks the status of all the processes that are matched by the filters.

Parameters:
  ProcNameFilters          : List containing the filters.
  NumberOfEachProcRequired : Specifies the number of processes required 
                             for each process type.
  ProcType                 : Type of the processes required.
                             The following process types are allowed. 
                                     PROC_TYPE_CHILDREN
                                     PROC_TYPE_PARENT
                                     PROC_TYPE_ALL
  LongCommand              : Flag to specify to use long or normal command 
                             to get the process IDs.

Return Value:
  Returns one of the monitor exit status: 
         ONLINE
         OFFLINE
         PARTIAL_ONLINE or 
         0 for error.
----------------------------------------------------------------------------

=cut

sub CheckProcProcessList {

  my ( $rlsProcNameFilters,
       $iNumberOfEachProcRequired,
       $ProcType,
       $bLongCommand ) = @_;

  my $sMyName = (caller 0)[3];

  LogInt::Message( LogInt::LLTrace,
  10100,
  "Proc",
  "CheckProcProcessList",
  "Subroutine <%s> called", $sMyName );

  if(!defined $bLongCommand) {
    $bLongCommand = 0;
  }
  else {
    $bLongCommand = 1;
  }

  #---------------------------------------------------------
  # get the 'ps' data then filter it using our ProcFilters.
  #---------------------------------------------------------
  my @lhrProcessDataList;

  if($bLongCommand == 0) { 
    #-----------------------------------
    # use old short ps command listing.
    #-----------------------------------
    if(GetProcessListHash(\@lhrProcessDataList) > 0) {
      #got more than zero proc's - continue
    } 
    else {
      LogInt::Message( LogInt::LLError, 
                       10101,
                       "Proc",
                       "CheckProcProcessList",
                       "GetProcessListHash returned no proc's");
      return 0; #exit with error 
    }
  }
  else {
    if(GetLongProcessListHash(\@lhrProcessDataList) > 0) {
      #got more than zero proc's - continue
    } 
    else {
      LogInt::Message( LogInt::LLError,
                       10102, 
                       "Proc",
                       "CheckProcProcessList",
                       "GetLongProcessListHash returned no proc's" );
      return 0; #exit with error 
    }
  }
  my @lhrMatchedProcessDataList = ();

  #------------------------------------------------
  # spin the passed in list of rlsProcNameFilters.
  #------------------------------------------------
  foreach my $sEachFilter (@$rlsProcNameFilters) {
    my @lhrTmpMatchedProcessDataList;
    if( FilterProcs( $sEachFilter, 
                     \@lhrProcessDataList, 
                     \@lhrTmpMatchedProcessDataList ) < $iNumberOfEachProcRequired ) {
      #--------------------------------------------------------------
      # we can jump here - regardless of anything else - we didnt 
      # even get iNumberOfEachProcRequired matches
      #--------------------------------------------------------------
      LogInt::Message( LogInt::LLTrace,
                       10103,
                       "Proc",
                       "CheckProcProcessList",
                       "Filter loop got less than [%s] matches returning status [OFFLINE]",
                       $iNumberOfEachProcRequired );
      return VCSDef_MonitorExit::OFFLINE;
    }

    if((PROC_TYPE_CHILDREN == $ProcType) || (PROC_TYPE_PARENT == $ProcType)) {
      my @lhrParents  = ();
      my @lhrChildren = ();
      SeparateOutChildrenAndParents(\@lhrTmpMatchedProcessDataList, \@lhrParents, \@lhrChildren);
      if(PROC_TYPE_CHILDREN == $ProcType) {
        if(@lhrChildren < $iNumberOfEachProcRequired) {
          LogInt::Message( LogInt::LLTrace,
                           10104,
                           "Proc",
                           "CheckProcProcessList",
                           "Filter loop got less than [%s] child matches returning status [OFFLINE]", 
                           $iNumberOfEachProcRequired );
          return VCSDef_MonitorExit::OFFLINE;
        }
      } 
      else {
        #-------------------------------------------------------------------------
        # because parents are defined in relation to children - if 
        # we have no parents we have no children
        # we already know that the FilterProcs found >= iNumberOfEachProcRequired 
        # we are taking a view that if there are no child-parent relations (because no parents)
        # then all entries will be taken as being parents - ie the test has passed
        #-------------------------------------------------------------------------
        if((@lhrParents != 0) || (@lhrParents < $iNumberOfEachProcRequired)) {
          LogInt::Message( LogInt::LLTrace,
                           10105,
                           "Proc",
                           "CheckProcProcessList",
                           "Filter loop got less than [%s] parent matches returning status [OFFLINE]", 
                           $iNumberOfEachProcRequired );
          return VCSDef_MonitorExit::OFFLINE;
        }
      }

    } #end if they are specifically matching on parent or child
  } #end of filter loop

  LogInt::Message( LogInt::LLTrace,
                   10106,
                   "Proc",
                   "CheckProcProcessList",
                   "Returning Status [ONLINE]" );

  return VCSDef_MonitorExit::ONLINE;
}


=pod

----------------------------------------------------------------------------
Function: Integer GetValuesForKeyFromProcDataHash(
                                          IN String Key, 
                                          IN ListOfHash Procs,
                                          OUT ListOfStrings Values )

Description:
  Get the list of values for the specified key from the array of processes.

Parameters:
  KeyForExtract       : Key type that needs to be extract from the process list.
  ProcInfoHashRefList : Array of processes that are used to extract.
  ValueList           : The list of entries that are extracted for the given key.

Return Value:
  Returns number entries of ValueList.
----------------------------------------------------------------------------

=cut

sub GetValuesForKeyFromProcDataHash {

  my ( $sKey,
       $rlhrProcs,  
       $rlsValues ) = @_;

  my $sMyName = (caller 0)[3];

  LogInt::Message( LogInt::LLTrace,
  10107,
  "Proc", 
  "GetValuesForKeyFromProcDataHash",
  "Subroutine <%s> called with:
  Key           [%s]", 
  $sMyName, $sKey );

  #----------------------------------------------------------
  # spin the hash array - extract just the "sKey" values an 
  # push into $rlsValues array
  # format of hash
  #
  # "UID"     =>  <user id>
  # "PID"     =>  <process id>
  # "PPID"    =>  <parent process id>
  # "COMMAND" =>  <command string>
  #----------------------------------------------------------

  foreach my $hrEachProcHashRef (@$rlhrProcs) {
    my $sTmpValue = $$hrEachProcHashRef{$sKey};
    push(@$rlsValues, $sTmpValue);
  } 

  my $iNumberEntries = @$rlsValues;
  LogInt::Message( LogInt::LLTrace,
                   10108,
                   "Proc",
                   "GetValuesForKeyFromProcDataHash",
                   "Got [%s] entries", $iNumberEntries);

  return $iNumberEntries;
}


=pod

----------------------------------------------------------------------------
Function: Void SeparateOutChildrenAndParents( IN ListOfHash Procs, 
            	                              OUT ListOfHash Parents, 
                        	              OUT ListOfHash Children )

Description:
  Separates the parent and children processes from the process list hash.

Parameters:
  Procs     : A hash containing the processes.
  Parents   : List of parent processes.
  Children  : List of children process IDs.

Return Value:
Void : No value is returned to the caller.
----------------------------------------------------------------------------

=cut

sub SeparateOutChildrenAndParents {

  my ( $rlhrProcs,
       $rlhrParents, 
       $rlhrChildren ) = @_;

  my $sMyName = (caller 0)[3];

  LogInt::Message( LogInt::LLTrace,
  10109,
  "Proc",
  "SeparateOutChildrenAndParents",
  "Subroutine <%s> called", $sMyName );
  #--------------------------------------------------------------
  # children can be parents and vice versa 
  # ie one entry could appear in both lists for separate reasons
  # for every pid (A) look at its ppid (A1)
  # if that ppid also exists as a pid (B) then they are child (A)
  # and parent (B) theres no doubt a cleaner way..
  #--------------------------------------------------------------

  #--------------------------------------------------------------
  # get 2 parallel arrays of the PID PPID
  # (didnt call GetValuesForKeyFromProcDataHash as we cant rely 
  # on the order and dont want to sort it)
  #--------------------------------------------------------------

  #--------------------------------------------------------------
  # format of hash
  #
  # "UID"     =>  <user id>
  # "PID"     =>  <process id>
  # "PPID"    =>  <parent process id>
  # "COMMAND" =>  <command string>
  #--------------------------------------------------------------
  my @liPIDList  = ();
  my @liPPIDList = ();

  foreach my $hrEachProcHashRef (@$rlhrProcs) {
    push(@liPIDList, $$hrEachProcHashRef{"PID"});
    push(@liPPIDList, $$hrEachProcHashRef{"PPID"});
  } #end for each proc

  #-------------------------------------------------------------------
  # for every pid (A) look at its ppid (AA) if that ppid (AA) also
  # exists as a pid (B) then they are child (A) and parent (B)
  #-------------------------------------------------------------------
  my %hCheckPIDHash = ();
  @hCheckPIDHash { @liPIDList } = (1) x @liPIDList;

  my @liRelativeChildPIDs  = ();
  my @liRelativeParentPIDs = ();

  #-----------------------
  # spin the PID array.
  #-----------------------
  my $iPIDListLength = @liPIDList;
  my $i = 0;
  while($i < $iPIDListLength) {
    my $sTmpPID_A = $liPIDList[$i];
    my $sTmpPPID_AA = $liPPIDList[$i];

    if($hCheckPIDHash{$sTmpPPID_AA}) {
      push(@liRelativeChildPIDs, $sTmpPID_A);
      push(@liRelativeParentPIDs, $sTmpPPID_AA);

      LogInt::Message( LogInt::LLTrace, 
                       10110,
                       "Proc", 
                       "SeparateOutChildrenAndParents",
                       "Found child PID_A [%s] parent PID_AA [%s]", 
                       $sTmpPID_A,
                       $sTmpPPID_AA );
    } 
    $i++;
  }

  #-------------------------------
  # put the results into hashs.
  #-------------------------------
  my %hCheckRelativeChildPIDHash = ();
  @hCheckRelativeChildPIDHash { @liRelativeChildPIDs } = (1) x @liRelativeChildPIDs;

  my %hCheckRelativeParentPIDHash = ();
  @hCheckRelativeParentPIDHash { @liRelativeParentPIDs } = (1) x @liRelativeParentPIDs;

  #--------------------------------------------------------------
  # spin the full hash array again - copying entires into their 
  # respective outgoing hash arrays.
  #--------------------------------------------------------------
  foreach my $hrEachProcHashRef (@$rlhrProcs) {
    my $iTmpPID = $$hrEachProcHashRef{"PID"};
    if($hCheckRelativeParentPIDHash{$iTmpPID}) {
     push(@$rlhrParents, $hrEachProcHashRef);
    }
    if($hCheckRelativeChildPIDHash{$iTmpPID}) {
     push(@$rlhrChildren, $hrEachProcHashRef);
    }
  } #end for each proc

  my $iRelativeParentCount = @$rlhrParents;
  my $iRelativeChildren    = @$rlhrChildren;
  LogInt::Message( LogInt::LLTrace,
                   10111, 
                   "Proc",
                   "SeparateOutChildrenAndParents",
                   "Got [%s] parents and [%s] children",
                   $iRelativeParentCount,
                   $iRelativeChildren );

  return;
}


=pod
                                                                                                                              
----------------------------------------------------------------------------
Function: Void KillPIDandAllChildren( IN Integer PID ) 

Description:
  Kills the process with the process ID and all its realated processes.
                                                                                                                              
Parameters:
  PID : Process ID to be killed.

Return Value:
  Void : No value is returned to the caller.
----------------------------------------------------------------------------
                                                                                                                              
=cut

sub KillPIDandAllChildren {

  my ( $iPID ) = @_ ;
  
  my $sMyName = (caller 0)[3];

  LogInt::Message( LogInt::LLTrace,
  10112,
  "KillPIDandAllChildren",
  "Proc",
  "Subroutine <%s> called with:
  PID          [%s]",
  $sMyName, $iPID );

  my %hRelatedPIDs   = ();
  my @lhProcListHash = ();
  my $iProcessCount  = Proc::GetLongProcessListHash( \@lhProcListHash );
  if( $iProcessCount == 0 ) {
    LogInt::Message( LogInt::LLError,
                     10113,
                     "KillPIDandAllChildren",
                     "Proc",
                     "Call to get process list did not return any processes" );
    return 0;
  }
                                                                                                                              
  #---------------------------------------------------
  # Initialize the hRelatedPIDs with the passed in PID
  #---------------------------------------------------
  foreach my $rhProcList ( @lhProcListHash ) {
    if( $rhProcList->{"PID"} == $iPID ) {
      $hRelatedPIDs{$iPID} = $rhProcList->{"COMMAND"};
      last;  # PIDs are garanteed to be unique
    }
  }
                                                                                                                              
  if( ! exists $hRelatedPIDs{$iPID} ) {
    LogInt::Message( LogInt::LLInfo,
                     10114,
                     "KillPIDandAllChildren",
                     "Proc",
                     "Process [%s] must have already died, nothing to kill...",
                     $iPID );
    return 0;
  }
                                                                                                                              
  #------------------------------------------------------------------
  # Get a list of all related processes to the poassed in one ($iPID)
  #------------------------------------------------------------------
  Proc::GetAllRelatedPIDs( $iPID, \@lhProcListHash, \%hRelatedPIDs );

  # Code change.. Atre 07 Aug 2008

  my ( $sKey, $sPidStr, @liGoners, @liUnDead ) = ();
  my %hiSignal = ( 'SIGTERM' => 15, 'SIGKILL' => 9 );

  @liGoners = keys %hRelatedPIDs; # Get PIDs to kill;                                

  if ( scalar @liGoners ) {
    foreach $sKey ( 'SIGTERM', 'SIGKILL' ) {
      #--------------------------
      # Send signal to each PID..
      #--------------------------
      if ( @liUnDead = grep {
                           kill ( 0, $_ ) &&                 # Valid PID..
                           kill ( $hiSignal { $sKey } , $_ ) # that could be delivered..
                       } @liGoners ) {                       # from what's found..

        map { $sPidStr .= "[$_]      [" . $hRelatedPIDs { $_ } . "]\n"; } @liUnDead;
        LogInt::Message ( LogInt::LLInfo,
                          10115,
                          "KillPIDandAllChildren",
                          "Proc",
                          "[%s] delivered to:
PID          CMD
----------------
%s----------------
Total: [%s]",             $sKey, $sPidStr, scalar @liUnDead );
      }
      else { # All killed..
        map { $sPidStr .= "[$_]      [" . $hRelatedPIDs { $_ } . "]\n"; } @liUnDead;
        LogInt::Message ( LogInt::LLTrace,
                          10116,
                          "KillPIDandAllChildren",
                          "Proc",
                          "[%s] NOT delivered to:
PID      CMD
----------------
%s----------------
Total: [%s]",             $sKey, $sPidStr, scalar @liUnDead );
      }
  
      $sPidStr = ""; # Truncte..
      sleep 2 if $sKey eq 'SIGTERM'; # Breather..
    }
  }

  return;
}


=pod

----------------------------------------------------------------------------
Function: Boolean LogKillCondition( IN Integer PIDtoKill,
                                    IN String CommandLine,
                                    IN Integer SignalNumber,
                                    IN String OSErrorString,
                                    IN String Message )
                                                                                                                              
Description:
  Logs the message returned by signaling the process. 
                                                                                                                              
Parameters:
  PIDtoKill     : The process ID that needs to be killed. 
  CommandLine   : The process command.
  SignalNumber  : Kill signal number sent to the process ID.
  OSErrorString : Error returned by the OS as a result of the kill signal.
  Message       : The result message after sending the signal to the process.

Return Value:
  Boolean value from the set of {0,1} where
        0 - false
        1 - true
----------------------------------------------------------------------------
                                                                                                                              
=cut

sub LogKillCondition {

  my ( $iPIDtoKill, 
       $sCommandLine,
       $iSignalNumber,
       $sOSErrorString,
       $sMessage ) = @_;
                                                                                                                              
  my $sMyName = (caller 0)[3];

  LogInt:Message( LogInt::LLTrace,
  10647,
  "LogKillCondition",
  "Proc",
  "Subroutine %s called with:
  PID              [%s]
  Command          [%s]
  Signal           [%s]
  Error            [%s]
  Message          [%s]",
  $sMyName, $iPIDtoKill, $sCommandLine,
  $iSignalNumber, $sOSErrorString, $sMessage );

  if( $sOSErrorString eq "No such process" || $sOSErrorString eq "Illegal seek" ) {
    LogInt:Message( LogInt::LLInfo,
    10647,
    "LogKillCondition",
    "Proc",
    "[%s] Process [%s] [%s] has already exited (OS message was [%s])",
    $$sMessage,
    $iPIDtoKill,
    $sCommandLine,
    $sOSErrorString );
  }
  else {
    LogInt( LogInt::LLInfo,
    10648,
    "LogKillCondition",
    "Proc",
    "[%s] Problems sending signal [%s] to process [%s] [%s] (OS Error was [%s])",
    $$sMessage,
    $iSignalNumber,
    $iPIDtoKill,
    $sCommandLine,
    $! );
  }

  return 1;
}


=pod
                                                                                                                              
----------------------------------------------------------------------------
Function: Boolean GetAllRelatedPIDs ( IN Integer PID,
                                      IN ArrayOfHashRefs ProcListHash,
                                      OUT Hash RelatedPIDs )

Description:
 Gives all the related process IDs for the given PID. 
                                                                                                                              
Parameters:
  PID          : Process ID. 
  ProcListHash : A hash containing the processes.
  RelatedPIDs  : List of related process IDs to PID.
                                                                                                                              
Return Value:
  Boolean value from the set of {0,1} where
        0 - false
        1 - true
----------------------------------------------------------------------------
                                                                                                                              
=cut

sub GetAllRelatedPIDs {

  my ( $iPID,
       $rlhrProcListHash,
       $rhRelatedPIDs ) = @_;
                                                                                                                              
  my $sMyName = (caller 0)[3];

  LogInt::Message( LogInt::LLTrace,
  10117,
  "Proc",
  "GetAllRelatedPIDs",
  "Subroutine <%s> called with:
  PID        [%s]",
  $sMyName, $iPID );

  foreach my $rhProcHash ( @$rlhrProcListHash ) {
    if( $iPID == $rhProcHash->{"PPID"} ) {
      #---------------
      # found a child!
      #---------------
      my $iChildPid = $rhProcHash->{"PID"};
      $rhRelatedPIDs->{$iChildPid} = $rhProcHash->{"COMMAND"};
                                                                                                                              
      #------------------------------------------------------
      # recurse and get any children of the child we've found
      #------------------------------------------------------
      my %hTmpChildPids = ();
      Proc::GetAllRelatedPIDs( $iChildPid, $rlhrProcListHash, \%hTmpChildPids );
                                                                                                                              
      #-------------------------------
      # add those children to the list
      #-------------------------------
      foreach my $iTmpPID ( keys %hTmpChildPids ) {
        $rhRelatedPIDs->{$iTmpPID} = $hTmpChildPids{$iTmpPID};
      }
    }
  }

  return 1;
}


=pod

----------------------------------------------------------------------------
Function: String ExtractToken( OUT String TokenToReturn, 
                               IN String OrigString, 
                               IN String Separator )

Description:
  Gives a token from the given string.

Parameters:
  TokenToReturn : Token extracted from the given string.
  OrigString    : Actual string from which token needs to be extracted.
  Separator     : Token separator.

Return Value:
  Returns the remaining string after the token is taken out.
----------------------------------------------------------------------------

=cut

sub ExtractToken {
 
  my ( $rsTokenToReturn,
       $sOrigString, 
       $sSeparator ) = @_;

  my $sMyName = (caller 0)[3];

  LogInt::Message( LogInt::LLTrace,
  10118,
  "Proc",
  "ExtractToken",
  "Subroutine <%s> called with:
  String            [%s]
  Separator         [%s]",
  $sMyName, $sOrigString, $sSeparator );

  #------------------------------------------------------------
  # used to parse the output from 'ps' GetProcessList
  # I wrote this - parsing routine from my perl irc client.
  #------------------------------------------------------------
  my $sTmpStr = $sOrigString;

  #------------------------------- 
  # skip any leading separators.
  #-------------------------------
  $sTmpStr =~ s/^$sSeparator+//;

  #-----------------------------------------------
  # get pos of first separator (if there is one).
  #-----------------------------------------------
  my $iPos = index($sTmpStr, $sSeparator);

  if($iPos < 0) {
    #------------------------------
    # no more tokens to extract.
    #------------------------------
    $$rsTokenToReturn = $sTmpStr;
    return ""; #return empty remaining string
  } 
  else {
    #---------------------------
    # split out the first token.
    #---------------------------
    ($$rsTokenToReturn) = split(/$sSeparator/, $sTmpStr);

    LogInt::Message( LogInt::LLTrace,
                     10119,
                     "Proc",
                     "ExtractToken",
                     "Token extracted [%s]", $$rsTokenToReturn);
    #----------------------------------------------
    # remove found token from remaining string.
    #----------------------------------------------
    my $sRemainingString = substr($sTmpStr, $iPos+1);

    #----------------------------
    # skip leading separator(s).
    #----------------------------
    $sRemainingString =~ s/^$sSeparator+//;

    LogInt::Message( LogInt::LLTrace,
                     10120,
                     "Proc",
                     "ExtractToken",
                     "String Remained [%s]", $sRemainingString );

    return $sRemainingString;
  }
}


=pod

 --------------------------------------------------------------------------
 Function : Boolean RemoveIPCResourcesForUser ( IN String User,
                                                IN Integer IpcFlags,
                                                IN RefListInteger Ids )

 Description : Removes IPC resources for specified UNIX user, using IPC IDs,
               if given. Else removes all IPCs for that user.
 
 Parameters :

 $sUser     : UNIX user whose IPC resources are to be removed
 $iIpcFlags : Behaviour flag
 $rliIds    : List of IPC IDs to remove for that user

 Values for Behaviour flag:

 O_IPCSHM : Shared Memory 
 O_IPCSEM : Semaphore
 O_IPCMSQ : Message Queue
 O_IPCALL : All of the above

 Return Value: boolean value in the set of {0,1} where
               0 false
               1 true
 --------------------------------------------------------------------------

=cut

sub RemoveIPCResourcesForUser {
  #----------------
  # Get Arguments..
  #----------------
  my ( $sUser,
       $iIpcFlags,
       $rliIds ) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  LogInt::Message( LogInt::LLTrace,
  10121,
  "Proc",
  "RemoveIPCResourcesForUser",
  "Subroutine <%s> called with:
   User      [%s]
   IPC Flags [%s]
   rliIds    [%s]",
   $sMyName, $sUser, $iIpcFlags, $rliIds );

  #-------------------------
  # Define local variables..
  #-------------------------
  my ( $iId, $iUid, $iIpcUid, $iRemoved, $iLocn, $iOwnerLocn, $sOwner,
       @lsTemp, @liShmIds, @liMqIds, @liSemIds, @liIdsRemoved,
       %hiIds, %hiStatFailed ) = ();

  my $sFormat  = 'l l l l ul ul i'; # Needed for unpack. Check intro(2)..
  my $lsVar    = 0;                 # Stores the binary structure semid_ds..
  my $sCommand = $::gsCommandIpcs;  # Get global..
  my $sErrorString = "";

  if ( $^O eq 'linux' ) {
    $iLocn = 1;  # Index locn. of ipc_perm.uid..
    $iOwnerLocn = 2; # "owner" index in ipcs output..
  }
  else {
    $iLocn = 0;  # Index locn. of ipc_perm.uid..
    $iOwnerLocn = 4; # "owner" index in ipcs output..
  }

  #-------------------------------------
  # Set return code to hopeful success..
  #-------------------------------------
  my $iReturnCode = 1;

  #--------------------
  # Validate the user..
  #--------------------
  unless ( defined ( $iUid = getpwnam $sUser ) ) {
    LogInt::Message ( LogInt::LLError,
    10122,
    "Proc",
    "RemoveIPCResourcesForUser",
    "<%s> Invalid User [%s] specfied. Cannot proceed.",
    $sMyName, $sUser );
    return 0; # Error..
  }

  #----------------
  # Check for IDs..
  #----------------
  if ( defined $rliIds && scalar @$rliIds ) {
    LogInt::Message ( LogInt::LLTrace,
    10123,
    "Proc",
    "RemoveIPCResourcesForUser",
    "<%s> Request to remove only [%s] IDs for User [%s]",                 
    $sMyName, join ( ' ', @$rliIds ), $sUser );

    %hiIds = map { $_ => 1 } @$rliIds; # Remove duplicates..
  }

  #-------------------------------
  # Read in the contents of IPCs..
  #-------------------------------
  foreach ( readpipe ( $sCommand ) ) {
    s/\n//; # Chomp..

    #-------------------------------------------------------
    # Attempt to skip IPC ids that definitely don't belong..
    #-------------------------------------------------------
    @lsTemp = $_ =~ /[^\s]+/g; # Split..
    $lsTemp [ 1 ] ||= "";
    $sOwner = $lsTemp [ $iOwnerLocn ] ||= ""; # Prevent undefs..

    if ( $lsTemp [ 1 ] =~ /^\d+/ && $sUser !~ /^$sOwner/ ) {
      LogInt::Message ( LogInt::LLTrace,
      10497,
      "Proc",
      "RemoveIPCResourcesForUser",
      "<%s> Skipping resource for owner [%s] Data [%s]",
      $sMyName, $sOwner, $_ );
      next;
    }
    elsif ( $sOwner =~ /\S/ ) {
      LogInt::Message ( LogInt::LLTrace,
      10498,
      "Proc",
      "RemoveIPCResourcesForUser",
      "<%s> Validating IPC resource for owner [%s] Data [%s]",
      $sMyName, $sOwner, $_ );
    }

    #---------------
    # Handle Linux..
    #---------------
    if ( $^O eq 'linux' ) {
      if ( $iIpcFlags & O_IPCSHM && # Shared Memory..
           /shmid/ ... /^$/ ) {
        /(\w+)\s+(\d+)/;
        push ( @liShmIds, $2 ) if defined $2;
      }
  
      if ( $iIpcFlags & O_IPCMSQ && # Message Queue..
           /semid/ ... /^$/ ) {      
        /(\w+)\s+(\d+)/;
        push ( @liSemIds, $2 ) if defined $2;
      }
  
      if ( $iIpcFlags & O_IPCSEM && # Semaphore..
           /msqid/ ... /^$/ ) {
        /(\w+)\s+(\d+)/;
        push ( @liMqIds, $2 ) if defined $2;
      }
    }
    else { # the rest..
      #--------------------
      # Skip removed IPCs..
      #--------------------
      if ( /^[m|s|q]\s*(\d+)\s+\w+\s+(\S)/ && # Of all three..
           defined $2                      && # Mode defined..
           ( $2 eq 'C' || $2 eq 'D') ) {      # Says, it's removed..
        next;                                 # Skip..
      }

      $iIpcFlags & O_IPCSHM && # Shared Memory..
      push ( @liShmIds, $_ =~ /^m\s*(\d+).*/ );
  
      $iIpcFlags & O_IPCMSQ && # Message Queue..
      push ( @liMqIds,  $_ =~ /^q\s*(\d+).*/ );
  
      $iIpcFlags & O_IPCSEM && # Semaphore..
      push ( @liSemIds, $_ =~ /^s\s*(\d+).*/ );
    }
  }

  #---------------------------------------------
  # Process each one separately as per request..
  #---------------------------------------------
 
  #----------------
  # Shared Memory..
  #----------------
  if ( $iIpcFlags & O_IPCSHM ) {
    foreach $iId ( @liShmIds ) {
      $lsVar = 0; # Clear..
      next if ( defined $rliIds && scalar @$rliIds && ! $hiIds { $iId } );

      unless ( shmctl ( $iId, IPC_STAT, $lsVar ) ) {
        LogInt::Message ( LogInt::LLError,
        10124,
        "Proc",
        "RemoveIPCResourcesForUser",
        "<%s> Failed to use shmctl(2) on SHM Id [%s]. Error [%s]",
        $sMyName, $iId, $! );
        push @{ $hiStatFailed { IPCSHM } }, $iId;
        next;
      }
 
      #-----------------------------------------------------
      # Unpack uid(2) out of the packed semid_ds structure..
      #-----------------------------------------------------
      $iIpcUid = ( unpack ( $sFormat, $lsVar ) ) [$iLocn];
  
      #-----------------------------
      # Remove resource if a match..
      #-----------------------------
      if ( $iIpcUid == $iUid ) {
        if ( shmctl ( $iId, IPC_RMID, 0 ) ) {
          push ( @liIdsRemoved, $iId );
          ++ $iRemoved; # Increment..
        }
        else {
          LogInt::Message( LogInt::LLError,
          10125,
          "Proc",
          "RemoveIPCResourcesForUser",
          "<%s> Failed to remove SHM Id [%s]. Error [%s]",
          $sMyName, $iId, $! );
          $iReturnCode = 0; # Failed..
        }
      }
    }
  }

  if ( scalar @liIdsRemoved ) {
    LogInt::Message( LogInt::LLTrace,
    10126,
    "Proc",
    "RemoveIPCResourcesForUser",
    "<%s> +--- Removed SHM Ids: [%s] for User [%s] Uid [%s]. Total [%s] ---+",
    $sMyName, 
    join ( ' ', @liIdsRemoved ),
    $sUser, 
    $iUid,
    scalar @liIdsRemoved ); 
    @liIdsRemoved = (); # Clear..
  }

  #----------------
  # Message Queue..
  #----------------
  if ( $iIpcFlags & O_IPCMSQ ) {
    foreach $iId ( @liMqIds ) {
      $lsVar = 0; # Clear..
      next if ( defined $rliIds && scalar @$rliIds && ! $hiIds { $iId } );

      unless ( msgctl ( $iId, IPC_STAT, $lsVar ) ) {
        LogInt::Message ( LogInt::LLError,
        10127,
        "Proc",
        "RemoveIPCResourcesForUser",
        "<%s> Failed to use msgctl(2) on Msg QId [%s]. Error [%s]", 
        $sMyName, $iId, $! );
        push @{ $hiStatFailed { IPCMSQ } }, $iId;
        next;
      }
  
      #-----------------------------------------------------
      # Unpack uid(2) out of the packed semid_ds structure..
      #-----------------------------------------------------
      $iIpcUid = ( unpack ( $sFormat, $lsVar ) ) [$iLocn];
  
      #-----------------------------
      # Remove resource if a match..
      #-----------------------------
      if ( $iIpcUid == $iUid ) {
        if ( msgctl ( $iId, IPC_RMID, 0 ) ) {
          push ( @liIdsRemoved, $iId );
          ++ $iRemoved; # Increment..
        }
        else {
          LogInt::Message( LogInt::LLError,
          10128,
          "Proc",
          "RemoveIPCResourcesForUser",
          "<%s> Failed to remove Msg QId [%s]. Error [%s]", 
          $sMyName, $iId, $! );
          $iReturnCode = 0; # Failed..
        }
      }
    }
  }

  if ( scalar @liIdsRemoved ) {
    LogInt::Message( LogInt::LLTrace,
    10129, 
    "Proc",
    "RemoveIPCResourcesForUser",
    "<%s> +--- Removed Msg QIds: [%s] for User [%s] Uid [%s]. Total [%s] ---+",
    $sMyName,
    join ( ' ', @liIdsRemoved ), 
    $sUser, 
    $iUid,
    scalar @liIdsRemoved );
    @liIdsRemoved = (); # Clear..
  }

  #------------
  # Semaphore..
  #------------
  if ( $iIpcFlags & O_IPCSEM ) {
    foreach $iId ( @liSemIds ) {
      $lsVar = 0; # Clear..
      next if ( defined $rliIds && scalar @$rliIds && ! $hiIds { $iId } );

      unless ( semctl ( $iId, 0, IPC_STAT, $lsVar ) ) {
        LogInt::Message( LogInt::LLError,
        10130,
        "Proc",
        "RemoveIPCResourcesForUser",
        "<%s> Failed to use semctl(2) on Sem Id [%s]. Error [%s]", 
        $sMyName, $iId, $! );
        push @{ $hiStatFailed { IPCSEM } }, $iId;
        next;
      }
  
      #-----------------------------------------------------
      # Unpack uid(2) out of the packed semid_ds structure..
      #-----------------------------------------------------
      $iIpcUid = ( unpack ( $sFormat, $lsVar ) ) [$iLocn];
  
      #-----------------------------
      # Remove resource if a match..
      #-----------------------------
      if ( $iIpcUid == $iUid ) {
        if ( semctl ( $iId, 0, IPC_RMID, 0 ) ) {
          push ( @liIdsRemoved, $iId );
          ++ $iRemoved; # Increment..
        }
        else {
          LogInt::Message( LogInt::LLError,
          10131,
          "Proc",
          "RemoveIPCResourcesForUser",
          "<%s> Failed to remove Sem Id [%s]. Error [%s]",
          $sMyName, $iId, $! );
          $iReturnCode = 0; # Failed..
        }
      }
    }
  }

  if ( scalar @liIdsRemoved ) {
    LogInt::Message( LogInt::LLTrace,
    10132,
    "Proc",
    "RemoveIPCResourcesForUser",
    "<%s> +--- Removed Sem Ids: [%s] for User [%s] Uid [%s]. Total [%s] ---+",
    $sMyName,
    join ( ' ', @liIdsRemoved ),
    $sUser,
    $iUid, 
    scalar @liIdsRemoved );
    @liIdsRemoved = (); # Clear..
  }

  #---------------------------------
  # Log, if we did anything useful..
  #---------------------------------
  if ( $iRemoved ) {
    LogInt::Message( LogInt::LLTrace,
    10133,
    "Proc",
    "RemoveIPCResourcesForUser",
    "<%s> Successfully removed [%s] IPC resources for User [%s] in all.",
    $sMyName, 
    $iRemoved,
    $sUser );
  }

  #-------------------------
  # Report on failed stats..
  #-------------------------
  if ( scalar ( @lsTemp = sort keys %hiStatFailed ) ) {

    foreach ( @lsTemp ) {
      $sErrorString .=
      "$_ => IDs [" . join ( " ", @{ $hiStatFailed { $_ } } ) . "]\n";
    }

    LogInt::Message( LogInt::LLError,
    10499,
    "Proc",
    "RemoveIPCResourcesForUser",
    "<%s> Following IDs could not be identified:
    %s
    Analyse and remove manually.",
    $sMyName, $sErrorString );

    $iReturnCode = 0; # Possibly failed..
  }

  return $iReturnCode;
}

1; # Because, Perl needs it.
