# $Copyright: Copyright (c) 2017 Veritas Technologies LLC.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# VERITAS TECHNOLOGIES LLC.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF VERITAS TECHNOLOGIES LLC.
#
# The  Licensed  Software  and  Documentation  are  deemed  to be commercial
# computer  software  as  defined  in  FAR  12.212 and subject to restricted
# rights  as  defined in FAR Section 52.227-19 "Commercial Computer Software
# - Restricted  Rights"  and  DFARS 227.7202,  et seq.  "Commercial Computer
# Software  and  Commercial Computer Software Documentation," as applicable,
# and any successor regulations, whether delivered by Veritas as on premises
# or  hosted  services.  Any  use,  modification,  reproduction  release,
# performance,  display  or  disclosure  of  the  Licensed  Software  and
# Documentation by the U.S. Government shall be solely in accordance with
# the terms of this Agreement. $
#---------------------------------------------------------------------------
# $Header: /project/isv-agent/ACCLib/lib/vcs/LogInt.pm,v 1.29 2014/07/18 16:55:19 lpatil Exp $
#
# Veritas Software Corporation
#
# Package Name: LogInt
#
# $Author: lpatil $
# $Date: 2014/07/18 16:55:19 $
# $Revision: 1.29 $
#
# Description: Internationalised logging (uses halog)
#
# Authors: 011787
#---------------------------------------------------------------------------

#-----------------------------------
# Register version of the module.
#-----------------------------------
use Version;
Version::TellVersion("\$Header: /project/isv-agent/ACCLib/lib/vcs/LogInt.pm,v 1.29 2014/07/18 16:55:19 lpatil Exp $::ETVHeader");


#-----------------------------------
# Declare package name.
#-----------------------------------
package LogInt;

#------------------------------------------
# Turn on strict for vars, subs and refs.
#------------------------------------------ 
use strict;

#-----------------------------------
# Include all the packages nedded.
#-----------------------------------
use ag_i18n_inc;

use IO::File;
use POSIX qw ( uname );
use POSIX qw ( strftime );
use Fcntl qw ( :DEFAULT :flock );

use Time::HiRes qw(time);

#--------------------------------------------------------------------
# Constant defines for logging levels (these should start at 1, 
# i.e. don't use zero)
#--------------------------------------------------------------------
use constant LLCritical    => 1;  # TAG_A = SEV_CRIT  C
use constant LLError       => 2;  # TAG_B = SEV_ERR   E
use constant LLWarn        => 3;  # TAG_C = SEV_WARN  W
use constant LLInfo        => 4;  # TAG_E = SEV_INFO  I
use constant LLTrace       => 5;  # TAG_D = SEV_NOTE  N
use constant LLTraceTrace  => 6;  # TAG_D = SEV_NOTE  N

#---------------------------------------------------------
# Set the default log threshold to level to "Info".
#---------------------------------------------------------
use constant VCS_ENGINE_DEFAULT_LOG_LEVEL => LogInt::LLInfo;

#-----------------------------------------------------
# Set the global logging level to default log level.
#-----------------------------------------------------
$::giLogIntLogLevel = VCS_ENGINE_DEFAULT_LOG_LEVEL;

my @lsOsDetails = uname ();
my $sOSType = $lsOsDetails [0];
my $iACCLib_CatId = 55000;

#----------------------------------
# Decide logging in Solaris zones..
#----------------------------------
my ( $bZone, $sOSVer, $sEP ) = ( 0, ( "" ) x 2 );
my %hsZoneVers = map { $_ => 1 } qw ( 5.10 5.11 ); # Keep extending Solaris zone OS..
( $sEP = $0 ) =~ s#.*/##; # Basename..

#--------------------------------
# Handle Solaris 10 local zones..
#--------------------------------
if ( $^O eq 'solaris' ) {
  $sOSVer = ( uname ) [2]; # Get OS version..
  if ( $sEP =~ /^(clean|monitor)$/ &&
       $hsZoneVers { $sOSVer }     &&
       ! kill ( 0, 1 ) ) { # Set flag..
    ++ $bZone;
  }
}

=pod

-----------------------------------------------------------------------
Function: void Message( IN Integer MessageLogLevel,
                        IN Integer MessageId,
                        IN String  AgentType,
                        IN String  EntryPoint,
                        IN String  MessageText,
                        IN ListOfStrings Params )

Description:
  Adds the message to the engine using halog if environment variable
  VCS_AGENTFW_USE_HALOG is set else it logs the message to standard 
  output.

  Optimizations:

  1. 27 Apr 2007.. Atre

  If message is of type TRACE/TRACETRACE, doesn't use halog. Logs the
  messages locally to /var/VRTSvcs/log/<Agent>_A.log

  2. 03 May 2007.. Atre

  Rotate logs A .. z based on size.

  3. 10 Jun 2010.. Atre

  Log locally for clean and monitor inside Solaris zones.

Parameters:
  MessageLogLevel : Logging level of the message. 
  MessageId       : Unique message ID of the message.
  AgentType       : Type of the agent.
  EntryPoint      : Name of the entrypoint.
  MessageText     : Actual message text to be logged.
  Params          : Parameters if any passed.

Return Value:
  void: No value is returned to the caller.
-----------------------------------------------------------------------

=cut

sub Message {

  my ( $iMessageLogLevel,
       $iMessageId,
       $sAgentType,
       $sEntryPoint,
       $sMessageText,
       @lsParams ) = @_;

  #-------------------------
  # Define local variables..
  #-------------------------
  my ( $bOpened, $iCurr, $iSize, $iReturnCode, $iCatId, $sClus, $sRes,
       $sEnt, $sAgent, $sLogFile, $sLockFile, $sHost, $sLogDir, $sOldLog, $sOSVer,
       $sLLHalogTag, $sHeader, $sTime, $sFile, $sFullMessageText, $sTemp, $sZoneFile,
       @lsDetails, %hsFiles ) = ();

  #----------------------------
  # Create local logging hash..
  #----------------------------
  my %hsSev = ( 'C' => 'CRITICAL', 'E' => 'ERROR', 'W' => 'WARNING',
                'N' => 'NOTICE',   'I' => 'INFO' );

  LogInt::GetLLHalogTag ( $iMessageLogLevel, \$sLLHalogTag );

  #---------------------------------------------------------
  # Add text to the start of the message text to get it 
  # into the right format.
  #---------------------------------------------------------
  my $sStandardMessageText = "";
  if( $sAgentType =~ /\S/ && $sEntryPoint =~ /\S/ ) {
    $sStandardMessageText = "$sAgentType:$sEntryPoint";
  }
  elsif ( $sAgentType =~ /\S/ ) {
    $sStandardMessageText = $sAgentType;
  }
  elsif ( $sEntryPoint =~ /\S/ ) {
    $sStandardMessageText = $sEntryPoint;
  }
    
  if  ( $sStandardMessageText =~ /\S/ ) {
    $sFullMessageText = "$sStandardMessageText:$sMessageText";    
  }
  else {
    $sFullMessageText = $sMessageText;    
  }

  #---------------------------------------------------------------
  # It's possible we have been passed undef params in the list
  # spin the array and swap undef's for "<undef>"
  #---------------------------------------------------------------
  grep { ! defined $_ && ( $_ = "<undef>" ) } @lsParams;
  my $sLogMessage = sprintf $sFullMessageText, @lsParams;
  # Write logs to <AgentName>_trigger_A.log file if VCSAG_EP_LOG_PIPE_FD is not defined
  # Since VCSAG_EP_LOG_PIPE_FD is defined by agent for entry point logs, & not defined in case of triggers, etc
  if (!defined($ENV{"VCSAG_EP_LOG_PIPE_FD"})) { # VCSAG_EP_LOG_PIPE_FD ENV variable is set from VCS 6.2 onwards
    if ( $iMessageLogLevel <= $::giLogIntLogLevel ) {
      if ( defined $ENV { VCS_AGENTFW_USE_HALOG } &&
           0 == $ENV { VCS_AGENTFW_USE_HALOG } ) {
        print "LogInt(stdout):TAG:$sLLHalogTag:$iMessageId $sLogMessage\n";
      } 
      else {
        #-----------------------------
        # Get environment attributes..
        #-----------------------------
        $iCatId = $iMessageId > 20000 ? $ENV { VCS_LOG_CATEGORY } : $iACCLib_CatId;
        $sAgent = $ENV { VCS_LOG_AGENT_NAME };
        $sRes   = $ENV { VCS_LOG_RESOURCE_NAME };
        $sEnt   = $ENV { VCS_LOG_SCRIPT_NAME };
  
        #----------------------------------------------
        # Support VAD/VCSOne/VCS 5.0.. Atre 30 Jan 2008
        #----------------------------------------------
        $sClus = ( $ENV { CLUSTER_HOME } ?
                   $ENV { CLUSTER_HOME } : $ENV { VCS_HOME } ) || "/opt/VRTSvcs";
  
        $sLogDir = "$VCSLOG/log";
        $sClus =~ s/.*(vad|vcs(|one))$/\U$1/; # VCS or VCSOne..
  
        $sHost = $lsOsDetails [1];
  
        #--------------
        # Log locally..
        #--------------
        $sLogFile = "$sLogDir/${sAgent}_trigger_A.log";
        $sLockFile = "$sLogDir/.${sAgent}_trigger.lock";
        $sOldLog = "$sLogDir/engine_B.log";
  
        #---------------------------
        # Re-cycle logs, if needed..
        #---------------------------
  
        #------------------
        # Get upper limit..
        #------------------
        $iSize = ( @lsDetails = stat $sOldLog ) ? $lsDetails [7] : 33554432; # 32 Mb.. Atre 28 Jul 2010
  
        #-----------------------
        # Get current log size..
        #-----------------------
        if ( ( $iCurr ) = ( stat ( $sLogFile ) ) [7] ) {
          $iCurr += length ( $sMessageText );
        }
  
        #-------------------
        # Create lock file..
        #-------------------
        if ( defined $iCurr && $iCurr >= $iSize ) {
          if ( sysopen ( FILDES, $sLockFile,
                         O_WRONLY|O_SYNC|O_TRUNC|O_CREAT|O_LARGEFILE, 0600 ) ) {
          
            #-------------------------
            # Acquire exclusive lock..
            #-------------------------
            if ( flock ( FILDES, LOCK_EX ) ) {
          
              #----------------------
              # Get latest log size..
              #----------------------
              if ( ( $iCurr ) = ( stat ( $sLogFile ) ) [7] ) {
                $iCurr += length ( $sMessageText );
              }
          
              #-----------------
              # Start renaming.. 
              #-----------------
              if ( defined $iCurr && $iCurr >= $iSize ) {
                %hsFiles = map {
                             $sLogDir . "/${sAgent}_trigger_" . chr ( $_ ) . ".log",
                             $sLogDir . "/${sAgent}_trigger_" . chr ( $_ + 1 ) . ".log",
                           } ( 65, 66 ); # A, B, and C only.. Atre 28 Jul 2010
  
                foreach $sFile ( sort { $b cmp $a } keys %hsFiles ) {
                  rename $sFile, $hsFiles { $sFile } if -e $sFile;
                }
  
                %hsFiles = (); # Release memory..
              }
          
  
              #------------------------------
              # Close file, to release lock..
              #------------------------------
              unless ( close ( FILDES ) ) {
                warn "LogInt(stderr): Failed to close [$sLockFile] after write: $!\n";
                return;
              }
  
              unlink ( $sLockFile ) if ( -e $sLockFile ); # Ignore error, for race..
            }
            else {
              warn "LogInt(stderr): Failed to acquire exclusive lock on [$sLockFile]: $!\n";
              return;
            }
          }
          else {
            warn "LogInt(stderr): Failed to open [$sLockFile] for write: $!\n";
            return;
          }
        }
  
        my $t = time;
        $sTime = strftime '%Y-%m-%dT%H:%M:%S', localtime $t;
        $sTime .= sprintf ".%03d", ($t-int($t))*1000;
        $sTime .= strftime '%z', localtime;
        substr($sTime, -2,0)=":";

        if ( sysopen ( FILDES, $sLogFile,
                       O_WRONLY|O_SYNC|O_EXCL|O_CREAT|O_LARGEFILE, 0600 ) ) {
          ++ $bOpened;

	  my @FQDN = gethostbyname($lsOsDetails[1]);

          $sHeader = "#
# Log Name:     $sAgent
# System:       $FQDN[0]
# SysInfo:      $lsOsDetails[0]:$lsOsDetails[1],$lsOsDetails[3],$lsOsDetails[2],$lsOsDetails[-1]
# Created:      $sTime
#";
          $sHeader =~ s/SunOS/Solaris/g;
  
          #---------------
          # Print header..
          #---------------
          print FILDES $sHeader, "\n\n";
        }
        elsif ( sysopen ( FILDES, $sLogFile,
                          O_WRONLY|O_SYNC|O_APPEND|O_LARGEFILE, 0600 ) ) {
          ++ $bOpened;
        }
        else {
          warn "LogInt(stderr): Failed to open [$sLogFile] for write [$!]\n";
        }
   
        #-----------------
        # Close log file..
        #-----------------
        if ( $bOpened ) {
          $bOpened = 0; # Reset..
          print FILDES "$sTime $sClus ", $hsSev { $sLLHalogTag },
          " V-16-$iCatId-$iMessageId Resource($sRes) ", "- ($sHost:$sEnt) $sLogMessage\n";
  
          unless ( close ( FILDES ) ) {
            warn "LogInt(stderr): Failed to close [$sLogFile] after write [$!]\n";
          }
        }
  
        #--------------------------
        # Enable fall back option..
        #--------------------------
        $sZoneFile = "$sLogDir/.${sAgent}.zone";
        $bZone = 0 if ( -e $sZoneFile ); # Reset global..
  
        #------------------------------------------
        # Log with engine only for Info and above..
        #------------------------------------------
        if ( ( $iMessageLogLevel < LLTrace ||
              -e "$sLogDir/.${sAgent}.eng" ) && ! $bZone ) {
          #--------------------------------------------------------------------
          # Now we need to spin the params list again - to surround each param 
          # with quotes, paramA becomes "paramA" this is to get round a problem 
          # in the halog call - it thinks we are trying to pass it options if 
          # one of out params happens to start with a '-'
          #--------------------------------------------------------------------
          my $sEncoding = "-encoding";
          if( $sOSType eq "Linux" || $sOSType eq "HP-UX" ) {
            $sEncoding .= " utf-8";
          }
          else {
            $sEncoding .= " utf8";
          }
  
          #-----------------------------------------------------------
          # Call the unified logging message call to log the message.
          #-----------------------------------------------------------
          VCSAG_LOG_MSG ( $sLLHalogTag,
                          $sLogMessage, 
                          $iMessageId, 
                          $sEncoding, 
                          @lsParams);
    
          if ( $iReturnCode = $? >> 8 ) {
            #-------------------------------------------------------------------
            # halog seems to have failed, just print the log message to stdout.
            #-------------------------------------------------------------------
            print "LogInt(halog call failed):TAG:$sLLHalogTag:$iMessageId $sLogMessage\n";
          }
        }
      }
    }
  
    $sTemp = "ffdc:$iMessageId:$sLogMessage\n";
    push ( @::gsFFDC, $sTemp );
  } 
  else { # Use the FFDC feature from VCS 6.2 onwards
    my $sEncoding = "-encoding";
    if( $sOSType eq "Linux" || $sOSType eq "HP-UX" ) {
      $sEncoding .= " utf-8";
    }
    else {
      $sEncoding .= " utf8";
    }
    if ( $iMessageLogLevel >= LLTrace) {
      VCSAG_LOGDBG_MSG(5, $sLogMessage); # DBG_5 log level is used for Trace messages
    } 
    else {
      VCSAG_LOG_MSG ( $sLLHalogTag,
                      $sLogMessage, 
                      $iMessageId, 
                      $sEncoding, 
                      @lsParams );
    }
  }
  return;
}


#----------------------------------------------------------
# These subroutines just call the main sub routine.
#----------------------------------------------------------

sub SetDebugLevelCritical   { setDebugLevel(LLCritical); }
sub SetDebugLevelError      { SetDebugLevel(LLError); }
sub SetDebugLevelWarn       { SetDebugLevel(LLWarn); }
sub SetDebugLevelInfo       { SetDebugLevel(LLInfo); }
sub SetDebugLevelTrace      { SetDebugLevel(LLTrace); }
sub SetDebugLevelTraceTrace { SetDebugLevel(LLTraceTrace); }


=pod

-----------------------------------------------------------------------
Function: void SetDebugLevel( IN Constant LogLevel )

Description:
  Sets the debug level to the specified level.

Parameters:
  LogLevel : Logging level of the message.

Return Value:
  void - No value is returned to the caller.
-----------------------------------------------------------------------

=cut

sub SetDebugLevel {

  my ( $iLogLevel ) = @_;

  #----------------------------------------------------------------------
  # if they are actually changing the level (i.e. not requesting the
  # level we are currently on)
  # log a message letting them know they changed the log level
  #----------------------------------------------------------------------

  if($::giLogIntLogLevel == $iLogLevel) {
    #they are requesting a change to the value we have already - ignore
  } 
  else {
    #they are requesting an actual change
    my $sLLString;
    LogInt::GetLLString($iLogLevel, \$sLLString);

    #--------------------------------------------------------------------------
    # log a notice (at the default level) that they are changing the log level.
    #--------------------------------------------------------------------------
    $::giLogIntLogLevel = $iLogLevel; # First make the change.. Atre 16 Apr 2007
    if ( $sLLString =~ /^(INFO|(TRACE){1,2})$/ ) {
      LogInt::Message ( $::giLogIntLogLevel,
                        10346,
                        "LogInt",
                        "SetDebugLevel",
                        "Information, Set Logging Level To [%s]", $sLLString );
    }

  }

  return;
}


=pod

-----------------------------------------------------------------------
Function: void SetDebugLevelWithString( IN String LogLevelString )

Description:
  Sets the Debug level with the specified log level string.

Parameters:
  LogLevelString : String indicating the logging level of the messages.

Return Value:
  void - No value is returned to the caller.
-----------------------------------------------------------------------

=cut

sub SetDebugLevelWithString {

  my ( $sLogLevelString ) = @_;

  #--------------------------------------------------------
  # validate the string, then call set log if it's valid.
  #--------------------------------------------------------

  my $iTmpLogLevel = undef;

  if(GetLLGivenString($sLogLevelString, \$iTmpLogLevel)) {
    SetDebugLevel($iTmpLogLevel);
  } 
  else {
    LogInt::Message( LLError,
                     10347,
                     "LogInt",
                     "SetDebugLevelWithString",
                     "Could not set the log level using string [%s]", $sLogLevelString );
  }
 
  return;
}


=pod

-----------------------------------------------------------------------
Function: void GetLLString( IN Constant MessageLogLevel,
                            OUT String LLString )

Description:
  Gives the logging level string for given log level ID.

Parameters:
  MessageLogLevel : Message logging level. Ex., LLWarn, LLError etc.
  LLString        : Corresponding message logging string. 
                    Ex., "WARN", "ERROR" etc. 

Return Value:
  void - No value is returned to the caller.
-----------------------------------------------------------------------

=cut

sub GetLLString {

  my ( $iMessageLogLevel, 
       $rsLLString ) = @_;

  #------------------------------------------------------------------
  # Build this dynamically since Perl 5.6.1 didn't like the old way.
  #------------------------------------------------------------------

  my %shLLtoStringLookup = ();

  $shLLtoStringLookup{LogInt::LLCritical}    = "CRITICAL";
  $shLLtoStringLookup{LogInt::LLError}       = "ERROR";
  $shLLtoStringLookup{LogInt::LLWarn}        = "WARN";
  $shLLtoStringLookup{LogInt::LLInfo}        = "INFO";
  $shLLtoStringLookup{LogInt::LLTrace}       = "TRACE";
  $shLLtoStringLookup{LogInt::LLTraceTrace}  = "TRACETRACE";

  if(exists $shLLtoStringLookup{$iMessageLogLevel}) {
    $$rsLLString = $shLLtoStringLookup{ $iMessageLogLevel };
  } 
  else {
    LogInt::Message( LLError,
                     10348, 
                     "LogInt", 
                     "GetLLString", 
                     "Invalid log level used [%s]", $iMessageLogLevel );

    $$rsLLString = "UNKNOWN";
  }
 
}


=pod

-----------------------------------------------------------------------
Function: void SetResourceName( IN String ResourceName )

Description:
  Sets global variable with the resource name given.

Parameters:
  ResourceName : Name of the resource which needs to be set.

Return Value:
  void : No value is returned to the caller.
-----------------------------------------------------------------------

=cut

sub SetResourceName {

  my ( $sResourceName ) = @_;

  my $sMyName = (caller 0)[3];

  LogInt::Message( LLError, 
  10349, 
  "LogInt",
  "SetResourceName", 
  "Subroutine <%s> called with:
  Resource Name          [%s]",
  $sMyName, $sResourceName );

  #-------------------------------
  # Set the Resource Name up.
  #-------------------------------
  if(defined $sResourceName) {
    $ENV{'VCS_LOG_RESOURCE_NAME'} = $sResourceName;
  } 
  else {
    #------------------
    # Set to unknown.
    #------------------
    $ENV{'VCS_LOG_RESOURCE_NAME'} = "UNKNOWN";
    LogInt::Message( LLError, 
                     10350, 
                     "LogInt",
                     "SetResourceName", 
                     "SetResourceName called with empty ResourceName" );
  }

  return;
}


#-----------------------------------------------------------------
# Return a logging level (const) for a string
# Can be one of "CRITICAL", "ERROR", "WARN", "INFO" or "TRACE" .
#-----------------------------------------------------------------

=pod

-----------------------------------------------------------------------
Function: Void GetLLGivenString( IN String LLString,
                                 OUT Constant MessageLogLevel )

Description:
  Gives message log level for a given log level string.

Parameters:
  LLString        : Log level string. Ex., WARN, ERROR, INFO etc
  MessageLogLevel : Message logging level for the message.
                    Ex., LLWarn, LLError etc.

Return Value:
  void - No value is returned to the caller.
-----------------------------------------------------------------------

=cut

sub GetLLGivenString {

  my ( $sLLString,
       $riMessageLogLevel ) = @_;

  my %shStringToLLLookup = ("CRITICAL"   => LogInt::LLCritical,
                            "ERROR"      => LogInt::LLError,
                            "WARN"       => LogInt::LLWarn,
                            "INFO"       => LogInt::LLInfo,
                            "TRACE"      => LogInt::LLTrace,
                            "TRACETRACE" => LogInt::LLTraceTrace,
                            );

  if(defined $sLLString) {
    if(exists $shStringToLLLookup{$sLLString}) {
      $$riMessageLogLevel = $shStringToLLLookup{$sLLString};
      return 1; #success
    }
  }

  return 0; #failed to lookup
}


=pod

-----------------------------------------------------------------------
Function: void IsValidLLString( IN String LLString )

Description:
  Checks the given log level string is valid or not.

Parameters:
  LLString : Log level String. Ex., WARN, ERROR, TRACE etc.

Return Value:
  void : No value is returned to the caller.
-----------------------------------------------------------------------

=cut

sub IsValidLLString {
  my ( $sLLString ) = @_;
  my $iMessageLogLevelTmp = undef;
  return GetLLGivenString($sLLString, \$iMessageLogLevelTmp);
}


=pod 

-----------------------------------------------------------------------
Function: void GetLLHalogTag( IN Constant MessageLogLevel,
                              OUT String LLHalogTag )

Description:
  Gives the tag for the logging message.

Parameters:
  MessageLogLevel : Message logging level. Ex., LLWarn, LLError etc.
  LLHalogTag      : Tag for the log level. Ex., A, B, C etc

Return Value:
  void - No value is returned to the caller.
-----------------------------------------------------------------------

=cut

sub GetLLHalogTag {

  my ( $iMessageLogLevel,
       $rsLLHalogTag ) = @_;

  #-----------------------------------------------------------------
  # Build this dynamically since Perl 5.6.1 didn't like the old way.
  #-----------------------------------------------------------------
  my %shLLtoHalogTagLookup = ();

  $shLLtoHalogTagLookup{LogInt::LLCritical}    = "C";
  $shLLtoHalogTagLookup{LogInt::LLError}       = "E";
  $shLLtoHalogTagLookup{LogInt::LLWarn}        = "W";
  $shLLtoHalogTagLookup{LogInt::LLInfo}        = "N";
  $shLLtoHalogTagLookup{LogInt::LLTrace}       = "I";
  $shLLtoHalogTagLookup{LogInt::LLTraceTrace}  = "I";

  if(exists $shLLtoHalogTagLookup{ $iMessageLogLevel }) {
    $$rsLLHalogTag = $shLLtoHalogTagLookup{ $iMessageLogLevel };
  } 
  else {
    LogInt::Message( LLError, 
                     10351,
                     "LogInt",
                     "GetLLHalogTag", 
                     "Invalid log level used [%s]", $iMessageLogLevel );

    $$rsLLHalogTag = $shLLtoHalogTagLookup{LogInt::LLWarn}; #use warn when loglevel unknown
  }
  return;
}

1;

